;**************************************************************************
;
;                           PVK40 Example : Display
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on B0,1,2,4 DISP on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
portb   equ     0xF81
trisb   equ     0xF93
portd   equ     0xF83
trisd   equ     0xF95
status  equ     0xFD8
wreg    equ     0xFE8
pcl     equ     0xFF9
; General Purpose Register definition :
tmp     equ     0x00
cnt1    equ     0x01            ;used for Wait subroutine
; Destination definition :
w       equ     0x00
f       equ     0x01
; RAM access bit definition :
a       equ     0x00            ;access RAM
banked  equ     0x01            ;bank specified by BSR register
; Bits definition :
#define DCOM    portb,0         ;display common anode
#define Z       status,2        ;Z is bit 2 in status register
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  setf    portd,a         ;set portd
        movlw   B'11111110'     ;pin RB0 is output
        movwf   trisb,a
;
Main:   setf    trisd,a         ;portd pins are inputs
        bsf     DCOM,a          ;turn off the display
        call    Wait            ;delay to stable inputs
        movf    portd,w,a       ;read portd (pushbuttons)
        bcf     DCOM,a          ;turn on the display
        clrf    trisd,a         ;portd pins are outputs
        movwf   tmp,a
;
        movlw   0x08
Test:   btfss   tmp,7,a         ;detect which button is pressed
        goto    Disp
        rlcf    tmp,f,a
        addlw   0xFF            ;subtract 1
        btfss   Z,a
        goto    Test
        movlw   0x0A            ;no button pressed, clear display
;
Disp:   call    Table           ;convert 0-0x0A to char 0-9
        movwf   portd,a         ;write to port
        call    Wait
        goto    Main
;--------------------------------------------------------------------------
; Subroutines :
Table:  rlncf   wreg,f,a        ;Display segments table
        addwf   pcl,f,a         ;wreg * 2 -> pcl
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1,a
Wait_B: decfsz  cnt1,f,a        ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;**************************************************************************
; end of PVK40 Example : Display
        end
