;**************************************************************************
;
;                           PVK40 Example : A/D Converter
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on PHOTO on S9
; 4) Switch on B3 LED on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P18f4520.INC>
; Bits definition :
#define LED     portb,3         ;LED is on the RB3 pin
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  bcf     LED,a
        movlw   B'11110111'     ;pin RB3 is output
        movwf   trisb,a
        movlw   B'00000101'     ;enable A/D converter module
        movwf   adcon0,a
        movlw   B'00000000'     ;pin RA1 is analog input
        movwf   adcon1,a
        movlw   B'00100100'     ;A/D Conversion Clock = FOSC/4
        movwf   adcon2,a        ;A/D Acquisition Time = 32 / FOSC
        movlw   B'10110001'     ;enable TMR1, 16bit - buffered
        movwf   t1con,a
;
Main:   bsf     adcon0,GO,a     ;start conversion
Adw:    btfsc   adcon0,GO,a     ;wait for A/D conversion to complete
        goto    Adw
;
        bsf     LED,a           ;turn on the LED
        movf    adresh,w,a      ;preset tmr1
        movwf   tmr1h,a
        clrf    tmr1l,a
        bcf     pir1,TMR1IF,a
T1w_A:  btfss   pir1,TMR1IF,a   ;wait for TMR1 overflow
        goto    T1w_A
;
        bcf     LED,a           ;turn off the LED
        movf    adresh,w,a      ;preset tmr1
        movwf   tmr1h,a
        clrf    tmr1l,a
        bcf     pir1,TMR1IF,a
T1w_B:  btfss   pir1,TMR1IF,a   ;wait for TMR1 overflow
        goto    T1w_B
        goto    Main            ;closes the loop
;**************************************************************************
; end of PVK40 Example : A/D Converter
        end
