;**************************************************************************
;
;                           PVK40 Example : A/D Converter 2
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on TRIM1 on S9
; 4) Switch on B0,1,2,4 DISP on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f877a.INC>
; General Purpose Register definition :
cnt1    equ     0x20            ;used in Wait subroutine
bckwreg equ     0x21            ;working register backup during interrupt
dig1    equ     0x22            ;digit '1'
dig2    equ     0x23            ;digit '10'
dig3    equ     0x24            ;digit '100'
dig4    equ     0x25            ;digit '1000'
param0  equ     0x26            ;used in Wto5d subroutine
param1  equ     0x27            ;used in Wto5d subroutine
param2  equ     0x28            ;used in Wto5d subroutine
param3  equ     0x29            ;used in Wto5d subroutine
param4  equ     0x2A            ;used in Wto5d subroutine
count   equ     0x2B            ;used in Wto5d subroutine
temp    equ     0x2C            ;used in Wto5d subroutine
intcnt  equ     0x2D            ;interrupt counting
; Bits definition :
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
#define DCOM3   portb,2         ;display common anode 3
#define DCOM4   portb,4         ;display common anode 4
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
        movwf   bckwreg         ;save wreg during interrupt
        bcf     intcon,T0IF     ;clear flag
        decfsz  intcnt,f        ;if intcnt > 1
        goto    Int_A           ;then return
;
        movlw   0x0B            ;else reinitialise intcnt register
        movwf   intcnt
        bsf     adcon0,GO       ;start A/D conversion
Adw:    btfsc   adcon0,GO       ;wait for A/D conversion to complete
        goto    Adw
        bsf     status,RP0      ;bank 1
        movf    adresl,W        ;copy adresh/l to param1/0
        bcf     status,RP0      ;bank 0
        movwf   param0
        movf    adresh,W
        movwf   param1
        call    Wto5d           ;convert to decimal number
        movf    param2,W        ;save result to dig1-dig4
        andlw   0x0F
        movwf   dig1
        swapf   param2,W
        andlw   0x0F
        movwf   dig2
        movf    param3,W
        andlw   0x0F
        movwf   dig3
        swapf   param3,W
        andlw   0x0F
        movwf   dig4
Int_A:  movf    bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   0xFF
        movwf   portb           ;set portb
        movwf   portd           ;set portd
        bsf     status,RP0      ;bank 1
        movlw   B'10000000'     ;pin RA2 is analog input
        movwf   adcon1
        movlw   B'11101000'     ;pins RB0,1,2,4 are outputs
        movwf   trisb
        clrf    trisd           ;portd pins are outputs
        movlw   B'11010110'     ;prescaler assigned to TMR0
        movwf   option_reg
        bcf     status,RP0      ;bank 0
        movlw   0x0A
        movwf   dig1
        movwf   dig2
        movwf   dig3
        movwf   dig4
        movlw   B'01010001'     ;enable A/D converter module
        movwf   adcon0
        movlw   0x0B            ;initialise intcnt register
        movwf   intcnt
        clrf    tmr0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
        bsf     intcon,GIE      ;enable global interrupt
;
Main:   movf    dig1,w
        call    Table
        movwf   portd
        bcf     DCOM1           ;turn on display 1
        call    Wait
        bsf     DCOM1           ;turn off display 1
        movf    dig2,w
        call    Table
        movwf   portd
        bcf     DCOM2           ;turn on display 2
        call    Wait
        bsf     DCOM2           ;turn off display 2
        movf    dig3,w
        call    Table
        movwf   portd
        bcf     DCOM3           ;turn on display 3
        call    Wait
        bsf     DCOM3           ;turn off display 3
        movf    dig4,w
        call    Table
        movwf   portd
        bcf     DCOM4           ;turn on display 4
        call    Wait
        bsf     DCOM4           ;turn off display 4
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  addwf   pcl,f           ;Display segments table
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1
Wait_B: decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;
Wto5d:  bcf     status,C        ;this subroutine converts 16-bit number
        clrf    count           ;in param0(LSB) and param1(MSB) to
        bsf     count,4         ;5-digit decimal number in param2-4
        clrf    param4
        clrf    param3
        clrf    param2
Wto5d2: rlf     param0,F
        rlf     param1,F
        rlf     param2,F
        rlf     param3,F
        rlf     param4,F
        decf    count,F
        btfsc   status,Z
        return
        movf    param2,W
        addlw   0x03
        movwf   temp
        btfsc   temp,3          ;test if result > 7
        movwf   param2
        movf    param2,W
        addlw   0x30
        movwf   temp
        btfsc   temp,7          ;test if result > 7
	movwf   param2
        movf    param3,W
        addlw   0x03
        movwf   temp
        btfsc   temp,3          ;test if result > 7
        movwf   param3
        movf    param3,W
        addlw   0x30
        movwf   temp
        btfsc   temp,7          ;test if result > 7
        movwf   param3
        movf    param4,W
        addlw   0x03
        movwf   temp
        btfsc   temp,3          ;test if result > 7
        movwf   param4
        movf    param4,W
        addlw   0x30
        movwf   temp
        btfsc   temp,7          ;test if result > 7
        movwf   param4
        goto    Wto5d2
;**************************************************************************
; end of PVK40 Example : A/D Converter 2
        end
