;**************************************************************************
;
;                           PVK40 Example : Analogue comparator
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on TRIM1 on S9
; 4) Switch on TRIM2 on S9
; 5) Switch on B0,1,2,4 DISP on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P18f4520.INC>
; General Purpose Register definition :
cnt1    equ     0x00            ;used in Wait subroutine
dig1    equ     0x01            ;digit '1'
dig2    equ     0x02            ;digit '10'
tmph    equ     0x03            ;used for comparsion
tmpl    equ     0x04            ;used for comparsion
; Bits definition :
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x08            ;interrupt vector
        bcf     intcon,T0IF,a   ;clear flag
        bsf     adcon0,GO,a     ;start A/D conversion
Adw1:   btfsc   adcon0,GO,a     ;wait for A/D conversion to complete
        goto    Adw1
        movff   adresl,tmpl     ;copy adresh/l to tmph/l
        movff   adresh,tmph
        bsf     adcon0,CHS0,a   ;analog channel = RA3
        bsf     adcon0,GO,a     ;start A/D conversion
Adw2:   btfsc   adcon0,GO,a     ;wait for A/D conversion to complete
        goto    Adw2
        bcf     adcon0,CHS0,a   ;analog channel = RA2
;
        movf    adresh,W,a      ;compare tmph/l with adresh/l
        subwf   tmph,W,a        ;tmph - adresh
        btfss   status,C,a
        bra     Dilo            ;tmph < adresh
        btfss   status,Z,a
        bra     Dihi            ;tmph > adresh
        movf    adresl,W,a
        subwf   tmpl,W,a        ;tmpl - adresl
        btfss   status,C,a
        bra     Dilo            ;tmpl < adresl
        btfss   status,Z,a
        bra     Dihi            ;tmpl > adresl
;
Dieq:   movlw   0x04            ;equal
        movwf   dig2,a
        movwf   dig1,a
        retfie  fast            ;fast return and set GIE
Dihi:   movlw   0x00            ;Higher
        movwf   dig2,a
        addlw   0x02
        movwf   dig1,a
        retfie  fast            ;fast return and set GIE
Dilo:   movlw   0x01            ;lower
        movwf   dig2,a
        addlw   0x02
        movwf   dig1,a
        retfie  fast            ;fast return and set GIE
;--------------------------------------------------------------------------
Start:  setf    portb,a         ;set portb
        setf    portd,a         ;set portd
        movlw   B'11111100'     ;pins RB0,1 are outputs
        movwf   trisb,a
        clrf    trisd,a         ;portd pins are outputs
        movlw   B'00001001'     ;enable A/D converter module
        movwf   adcon0,a
        movlw   B'00001011'     ;pins RA2,3 are analog inputs
        movwf   adcon1,a
        movlw   B'10100100'     ;A/D Conversion Clock = FOSC/4
        movwf   adcon2,a        ;A/D Acquisition Time = 32 / FOSC
        movlw   B'10000001'     ;prescaler 1:4 assigned to TMR0
        movwf   t0con,a
        clrf    tmr0h,a
        clrf    tmr0l,a
        movlw   0x05
        movwf   dig1,a
        movwf   dig2,a
        bcf     rcon,IPEN,a     ;disable priority interrupt
        movlw   B'00100000'     ;unmask interrupt from TMR0
        movwf   intcon,a
        bsf     intcon,GIE,a    ;enable global interrupt
;
Main:   movf    dig1,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM1,a         ;turn on display 1
        call    Wait
        call    Wait
        bsf     DCOM1,a         ;turn off display 1
        movf    dig2,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM2,a         ;turn on display 2
        call    Wait
        call    Wait
        bsf     DCOM2,a         ;turn off display 2
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  rlncf   wreg,f,a        ;Display segments table
        addwf   pcl,f,a         ;wreg * 2 -> pcl
        retlw   B'10001001'     ;H
        retlw   B'11000111'     ;L
        retlw   B'11101111'     ;i
        retlw   B'10100011'     ;o
        retlw   B'10111111'     ;-
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1,a
Wait_B: decfsz  cnt1,f,a        ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;**************************************************************************
; end of PVK40 Example : Analogue comparator
        end
