;**************************************************************************
;
;                      PVK40 Example : Serial EEPROM 93xx (SW)
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; On your PVK40 board :
; 1) Insert 93LC46B into 93xx socket
; 2) Switch off JP1 jumper
; 3) Switch off all DIP switches except:
; 4) Switch on OSC 3.276M on S9
; 5) Switch on B0,1,2,4 DISP on S11
; 6) Switch on B3 LED on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P18f4520.INC>
; General Purpose Register definition :
cnt1    equ     0x00            ;used in Wait subroutine
dig1    equ     0x01            ;digit '1'
dig2    equ     0x02            ;digit '10'
dig3    equ     0x03            ;digit '100'
dig4    equ     0x04            ;digit '1000'
eedat0  equ     0x05            ;EEPROM data 0 (LSB)
eedat1  equ     0x06            ;EEPROM data 1 (MSB)
eeadrs  equ     0x07            ;used in Wree and Rdee subroutines
digtmp1 equ     0x08            ;store dig1 value
digtmp2 equ     0x09            ;store dig2 value
swrtmp0 equ     0x0A            ;used in Swree subroutine
swrtmp1 equ     0x0B            ;used in Swree subroutine
buffer  equ     0x0C            ;used in Output subroutine
bufcnt  equ     0x0D            ;used in Output subroutine
; Bits definition :
#define EECS    porta,5         ;EEPROM chip select
#define EEDI    portc,4         ;EEPROM data in
#define EEDO    portc,5         ;EEPROM data out
#define EECLK   portc,3         ;EEPROM clock
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
#define DCOM3   portb,2         ;display common anode 3
#define DCOM4   portb,4         ;display common anode 4
#define LED     portb,3         ;LED
#define PB0     portd,0         ;pushbutton 0
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x08            ;interrupt vector
        bcf     intcon,T0IF,a   ;clear flag
        incf    dig1,f,a        ;increment dig1
        movlw   0x09
        cpfsgt  dig1,a
        bra     Int_A
        clrf    dig1,a          ;if dig1 > 9 then dig1 = 0
        incf    dig2,f,a        ;and increment dig2
Int_A:  movlw   0x09
        cpfsgt  dig2,a
        bra     Int_B
        clrf    dig2,a          ;if dig2 > 9 then dig2 = 0
        incf    dig3,f,a        ;and increment dig3
Int_B:  movlw   0x09
        cpfsgt  dig3,a
        bra     Int_C
        clrf    dig3,a          ;if dig3 > 9 then dig3 = 0
        incf    dig4,f,a        ;and increment dig4
Int_C:  movlw   0x0A
        cpfslt  dig4,a
        clrf    dig4,a
        retfie  fast            ;fast return and set GIE
;--------------------------------------------------------------------------
Start:  clrf    porta,a         ;clear porta latch
        clrf    portc,a         ;clear portc latch
        movlw   B'11110111'     ;clear RB3 pin latch
        movwf   portb,a
        setf    portd,a         ;set portd latch
        movlw   B'00001111'     ;porta pins are digital
        movwf   adcon1,a
        movlw   B'11011111'     ;pin RA5 is output
        movwf   trisa,a
        movlw   B'11100000'     ;pins RB0,1,2,3,4 are outputs
        movwf   trisb,a
        movlw   B'11010111'     ;pins RC3,5 are outputs
        movwf   trisc,a
        clrf    trisd,a         ;portd pins are outputs
;
        movlw   0x00            ;read address 0
        call    Rdee            ;from EEPROM
        movf    eedat1,w,a
        andlw   0x0F
        movwf   dig4,a          ;save to dig4
        movf    eedat0,w,a
        andlw   0x0F
        movwf   dig3,a          ;save to dig3
        movlw   0x01            ;read address 1
        call    Rdee            ;from EEPROM
        movf    eedat1,w,a
        andlw   0x0F
        movwf   dig2,a          ;save to dig2
        movf    eedat0,w,a
        andlw   0x0F
        movwf   dig1,a          ;save to dig1
        call    Ewen            ;EEPROM write enable
;
        movlw   B'10000001'     ;prescaler 1:4 assigned to TMR0
        movwf   t0con,a
        clrf    tmr0h,a
        clrf    tmr0l,a
        bcf     rcon,IPEN,a     ;disable priority interrupt
        movlw   B'00100000'     ;unmask interrupt from TMR0
        movwf   intcon,a
        bsf     intcon,GIE,a    ;enable global interrupt
;
Main:   movf    dig1,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM1,a         ;turn on display 1
        call    Wait
        bsf     DCOM1,a         ;turn off display 1
        movf    dig2,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM2,a         ;turn on display 2
        call    Wait
        bsf     DCOM2,a         ;turn off display 2
        movf    dig3,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM3,a         ;turn on display 3
        call    Wait
        bsf     DCOM3,a         ;turn off display 3
        movf    dig4,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM4,a         ;turn on display 4
        call    Wait
        bsf     DCOM4,a         ;turn off display 4
;
        bsf     trisd,0,a       ;RD0 is input
        call    Wait            ;delay to stable inputs
        btfss   PB0,a           ;if pushbutton is pressed
        bsf     LED,a           ;turn on the LED
        btfsc   PB0,a
        bcf     LED,a
        bcf     trisd,0,a       ;RD0 is output
;
        btfss   LED,a           ;if the LED is on
        goto    Main
        bcf     intcon,GIE,a    ;disable interrupt
        movff   dig1,digtmp1    ;save dig1-dig4 state
        movff   dig2,digtmp2
        movff   dig3,eedat0
        movff   dig4,eedat1
        bsf     intcon,GIE,a    ;enable interrupt
        movlw   0x00
        call    Swree           ;write to EEPROM
        movff   digtmp1,eedat0
        movff   digtmp2,eedat1
        movlw   0x01
        call    Swree           ;write to EEPROM
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  andlw   0x0F            ;Display segments table
        bsf     pclath,0,a      ;subroutine position = 0x0001xx
        rlncf   wreg,f,a        ;wreg * 2 -> pcl
        addwf   pcl,f,a
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1,a
Wait_B: decfsz  cnt1,f,a        ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;
Output: movwf   buffer,a        ;place data in buffer
        movlw   0x08
        movwf   bufcnt,a
Outpt2: btfss   buffer,7,a      ;EEDO := buffer MSb
        bcf     EEDO,a
        btfsc   buffer,7,a
        bsf     EEDO,a
        bsf     EECLK,a         ;EECLK = 1
        nop
        bcf     EECLK,a         ;EECLK = 0
        btfss   EEDI,a          ;buffer MSb := EEDI
        bcf     buffer,7,a
        btfsc   EEDI,a
        bsf     buffer,7,a
        rlncf   buffer,f,a
        decfsz  bufcnt,f,a
        goto    Outpt2
        return
;
Ewen:   bsf     EECS,a          ;set chip select line high
        movlw   B'00000001'     ;start bit
        call    Output
        movlw   B'00110000'     ;opcode for EWEN
        call    Output
        bcf     EECS,a          ;clear chip select line
        return
;
Wree:   bsf     EECS,a          ;set chip select line high
        movwf   eeadrs,a
        movlw   B'00000001'     ;start bit
        call    Output
        movf    eeadrs,w,a      ;opcode + addr. for WRITE
        andlw   B'00111111'
        iorlw   B'01000000'
        call    Output
        movf    eedat1,w,a      ;data 15-8
        call    Output
        movf    eedat0,w,a      ;data 7-0
        call    Output
        bcf     EECS,a          ;clear chip select line
        nop                     ;EECS must be low for > 250 ns
        bsf     EECS,a          ;set chip select line high
Wreew:  btfss   EEDI,a          ;test ready/busy status
        goto    Wreew
        bcf     EECS,a          ;clear chip select line
        return
;
Rdee:   bsf     EECS,a          ;set chip select line high
        movwf   eeadrs,a
        movlw   B'00000001'     ;start bit
        call    Output
        movf    eeadrs,w,a      ;opcode + addr. for READ
        andlw   B'00111111'
        iorlw   B'10000000'
        call    Output
        movlw   0x00
        call    Output          ;8x clk
        movff   buffer,eedat1   ;read received data, copy to eedat1
        movlw   0x00
        call    Output          ;8x clk
        movff   buffer,eedat0   ;read received data, copy to eedat0
        bcf     EECS,a          ;clear chip select line
        return
;
Swree:  movwf   eeadrs,a        ;this subroutine writes to EEPROM
        movff   eedat0,swrtmp0  ;use this subroutine to avoid multiply
        movff   eedat1,swrtmp1  ;rewriting one address with the same value
        movf    eeadrs,w,a
        call    Rdee            ;read EEPROM data
        movf    eedat0,w,a      ;compare data in EEPROM
        cpfseq  swrtmp0,a       ;with data to write
        bra     Swreew
        movf    eedat1,w,a
        cpfseq  swrtmp1,a
        bra     Swreew
        return
Swreew: movff   swrtmp0,eedat0
        movff   swrtmp1,eedat1
        movf    eeadrs,w,a
        call    Wree            ;write to EEPROM
        return
;**************************************************************************
; end of PVK40 Example : Serial EEPROM 93xx (SW)
        end
