;**************************************************************************
;
;                           PVK40 Example : Thermometer
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on TEMP on S9
; 4) Switch on B0,1,2,4 DISP on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P18f4520.INC>
; General Purpose Register definition :
cnt1    equ     0x00            ;used in Wait subroutine
dig1    equ     0x01            ;digit '1'
dig2    equ     0x02            ;digit '10'
dig3    equ     0x03            ;digit '100'
dig4    equ     0x04            ;digit '1000'
param0  equ     0x05            ;used in Wto5d subroutine
param1  equ     0x06            ;used in Wto5d subroutine
param2  equ     0x07            ;used in Wto5d subroutine
param3  equ     0x08            ;used in Wto5d subroutine
param4  equ     0x09            ;used in Wto5d subroutine
count   equ     0x0A            ;used in Wto5d subroutine
temp    equ     0x0B            ;used in Wto5d subroutine
; Bits definition :
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
#define DCOM3   portb,2         ;display common anode 3
#define DCOM4   portb,4         ;display common anode 4
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x08            ;interrupt vector
        bcf     intcon,T0IF,a   ;clear flag
        bsf     adcon0,GO,a     ;start A/D conversion
Adw:    btfsc   adcon0,GO,a     ;wait for A/D conversion to complete
        goto    Adw
        movf    adresl,W,a      ;copy adresh/l to param1/0
        addlw   0x0A            ;correction constant
        movwf   param0,a
        movlw   0x00
        addwfc  adresh,W,a
        movwf   param1,a
        call    Wto5d           ;convert to decimal number
        movf    param2,W,a      ;save result to dig3-dig4
        andlw   0x0F
        movwf   dig3,a
        swapf   param2,W,a
        andlw   0x0F
        movwf   dig4,a
        movlw   0x0B
        movwf   dig2,a
        movlw   0x0C
        movwf   dig1,a
        retfie  fast            ;fast return and set GIE
;--------------------------------------------------------------------------
Start:  setf    portb,a         ;set portb
        setf    portd,a         ;set portd
        movlw   B'00000001'     ;enable A/D converter module
        movwf   adcon0,a
        movlw   B'00001110'     ;pin RA0 is analog input
        movwf   adcon1,a
        movlw   B'10001100'     ;A/D Conversion Clock = FOSC/4
        movwf   adcon2,a        ;A/D Acquisition Time = 8 / FOSC
        movlw   B'11101000'     ;pins RB0,1,2,4 are outputs
        movwf   trisb,a
        clrf    trisd,a         ;portd pins are outputs
;
        movlw   0x0A
        movwf   dig1,a
        movwf   dig2,a
        movwf   dig3,a
        movwf   dig4,a
;
        movlw   B'10000001'     ;prescaler 1:4 assigned to TMR0
        movwf   t0con,a
        clrf    tmr0h,a
        clrf    tmr0l,a
        bcf     rcon,IPEN,a     ;disable priority interrupt
        movlw   B'00100000'     ;unmask interrupt from TMR0
        movwf   intcon,a
        bsf     intcon,GIE,a    ;enable global interrupt
;
Main:   movf    dig1,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM1,a         ;turn on display 1
        call    Wait
        bsf     DCOM1,a         ;turn off display 1
        movf    dig2,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM2,a         ;turn on display 2
        call    Wait
        bsf     DCOM2,a         ;turn off display 2
        movf    dig3,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM3,a         ;turn on display 3
        call    Wait
        bsf     DCOM3,a         ;turn off display 3
        movf    dig4,w,a
        call    Table
        movwf   portd,a
        bcf     DCOM4,a         ;turn on display 4
        call    Wait
        bsf     DCOM4,a         ;turn off display 4
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  andlw   0x0F            ;Display segments table
        rlncf   wreg,f,a        ;wreg * 2 -> pcl
        addwf   pcl,f,a
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
        retlw   B'10011100'     ;degree
        retlw   B'11000110'     ;C
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0xFF            ;this subroutine wait 770 cycles
        movwf   cnt1,a
Wait_B: decfsz  cnt1,f,a        ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;
Wto5d:  bcf     status,C,a      ;this subroutine converts 16-bit number
        clrf    count,a         ;in param0(LSB) and param1(MSB) to
        bsf     count,4,a       ;5-digit decimal number in param2-4
        clrf    param4,a
        clrf    param3,a
        clrf    param2,a
Wto5d2: rlcf    param0,f,a
        rlcf    param1,f,a
        rlcf    param2,f,a
        rlcf    param3,f,a
        rlcf    param4,f,a
        decf    count,f,a
        btfsc   status,Z,a
        return
        movf    param2,W,a
        addlw   0x03
        movwf   temp,a
        btfsc   temp,3,a        ;test if result > 7
        movwf   param2,a
        movf    param2,W,a
        addlw   0x30
        movwf   temp,a
        btfsc   temp,7,a        ;test if result > 7
	movwf   param2,a
        movf    param3,W,a
        addlw   0x03
        movwf   temp,a
        btfsc   temp,3,a        ;test if result > 7
        movwf   param3,a
        movf    param3,W,a
        addlw   0x30
        movwf   temp,a
        btfsc   temp,7,a        ;test if result > 7
        movwf   param3,a
        movf    param4,W,a
        addlw   0x03
        movwf   temp,a
        btfsc   temp,3,a        ;test if result > 7
        movwf   param4,a
        movf    param4,W,a
        addlw   0x30
        movwf   temp,a
        btfsc   temp,7,a        ;test if result > 7
        movwf   param4,a
        goto    Wto5d2
;**************************************************************************
; end of PVK40 Example : Thermometer
        end
