;**************************************************************************
;
;                           PVK40 Example : Sound-level-meter
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on E0 SEGM9 on S9
; 4) Switch on E1 SEGM10 on S9
; 5) Switch on A5 MIC on S9
; 6) Switch on C1 BARGRAPH on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f877a.INC>
; General Purpose Register definition :
smpcnt  equ     0x20            ;used in Sndm subroutine
peak1   equ     0x21            ;maximum, LSB
peak2   equ     0x22            ;maximum, MSB
down1   equ     0x23            ;minimum, LSB
down2   equ     0x24            ;minimum, MSB
cnst1   equ     0x25            ;constant, LSB
cnst2   equ     0x26            ;constant, MSB
lat1    equ     0x27            ;used in Subtr subroutine
lat2    equ     0x28            ;used in Subtr subroutine
temp    equ     0x29            ;Subtr subroutine result
bargr1  equ     0x2A            ;bargraph data, LSB
bargr2  equ     0x2B            ;bargraph data, MSB
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  clrf    portc           ;clear portc latch
        movlw   0xFF
        movwf   portd           ;set portd latch
        movwf   porte           ;set porte latch
        bsf     status,RP0      ;bank 1
        movlw   B'10000010'     ;pin RA5 is analog input,
        movwf   adcon1          ;porte pins are digital
        movlw   B'11111101'     ;pin RC1 is output
        movwf   trisc
        clrf    trisd           ;portd pins are outputs
        movlw   B'00001100'     ;pins RE0,1 are outputs
        movwf   trise
        movlw   B'11010000'     ;prescaler assigned to TMR0
        movwf   option_reg      ;TMR0 period = 0.625 ms
        bcf     status,RP0      ;bank 0
        movlw   B'01100001'     ;enable A/D converter module
        movwf   adcon0
        clrf    tmr0
        bcf     intcon,T0IF     ;clear timer0 overflow flag
;
Main:   call    Sndm            ;measure maximum and minimum value
        movf    down1,W         ;peak2:peak1 - down2:down1 to peak2:peak1
        subwf   peak1,F
        btfss   status,C
        decf    peak2,F
        movf    down2,W
        subwf   peak2,F
;
        movlw   0xFF
        movwf   bargr1
        movwf   bargr2
;
        movlw   0x2D            ;cnst2:1 = 45
        movwf   cnst1	
        movlw   0x00
        movwf   cnst2           ;if peak2:1 > cnst
        call    Subtr
        movwf   temp
        btfsc   temp,0
        bcf     bargr1,0        ;switch on bargraph segment 1
;
        movlw   0x40            ;cnst2:1 = 64
        movwf   cnst1	
        movlw   0x00
        movwf   cnst2           ;if peak2:1 > cnst
        call    Subtr
        movwf   temp
        btfsc   temp,0
        bcf     bargr1,1        ;switch on bargraph segment 2
;
        movlw   0x5B            ;cnst2:1 = 91
        movwf   cnst1	
        movlw   0x00
        movwf   cnst2           ;if peak2:1 > cnst
        call    Subtr
        movwf   temp
        btfsc   temp,0
        bcf     bargr1,2        ;switch on bargraph segment 3
;
        movlw   0x80            ;cnst2:1 = 128
        movwf   cnst1	
        movlw   0x00
        movwf   cnst2           ;if peak2:1 > cnst
        call    Subtr
        movwf   temp
        btfsc   temp,0
        bcf     bargr1,3        ;switch on bargraph segment 4
;
        movlw   0xB5            ;cnst2:1 = 181
        movwf   cnst1	
        movlw   0x00
        movwf   cnst2           ;if peak2:1 > cnst
        call    Subtr
        movwf   temp
        btfsc   temp,0
        bcf     bargr1,4        ;switch on bargraph segment 5
;
        movlw   0x00            ;cnst2:1 = 256
        movwf   cnst1	
        movlw   0x01
        movwf   cnst2           ;if peak2:1 > cnst
        call    Subtr
        movwf   temp
        btfsc   temp,0
        bcf     bargr1,5        ;switch on bargraph segment 6
;
        movlw   0x39            ;cnst2:1 = 313
        movwf   cnst1	
        movlw   0x01
        movwf   cnst2           ;if peak2:1 > cnst
        call    Subtr
        movwf   temp
        btfsc   temp,0
        bcf     bargr1,6        ;switch on bargraph segment 7
;
        movlw   0x00            ;cnst2:1 = 512
        movwf   cnst1	
        movlw   0x02
        movwf   cnst2           ;if peak2:1 > cnst
        call    Subtr
        movwf   temp
        btfsc   temp,0
        bcf     bargr1,7        ;switch on bargraph segment 8
;
        movlw   0x80            ;cnst2:1 = 640
        movwf   cnst1	
        movlw   0x02
        movwf   cnst2           ;if peak2:1 > cnst
        call    Subtr
        movwf   temp
        btfsc   temp,0
        bcf     bargr2,0        ;switch on bargraph segment 9
;
        movlw   0xFA            ;cnst2:1 = 762
        movwf   cnst1	
        movlw   0x02
        movwf   cnst2           ;if peak2:1 > cnst
        call    Subtr
        movwf   temp
        btfsc   temp,0
        bcf     bargr2,1        ;switch on bargraph segment 10
;
        movf    bargr2,W        ;write to bargraph
        movwf   porte
        movf    bargr1,W
        movwf   portd
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Sndm:   clrf    peak1           ;detect maximum and minimum value
        clrf    peak2
        movlw   0xFF
        movwf   down1
        movwf   down2
        movlw   0xFF            ;255 samples
        movwf   smpcnt
Sndm_A: bsf     adcon0,GO       ;starts A/D conversion
Sndm_B: btfsc   adcon0,GO       ;wait for A/D conversion to complete
        goto    Sndm_B
        btfss   intcon,T0IF     ;wait for TMR0 overflow
        goto    Sndm_B
        bcf     intcon,T0IF     ;clear timer0 overflow flag
;
        movf    adresh,w
        subwf   peak2,w
        btfss   status,Z        ;if adresh <> peak2
        goto    Sndm_C          ;skip to Sndm_C
        bsf     status,RP0      ;bank 1
        movf    adresl,W        ;else test adresl
        bcf     status,RP0      ;bank 0
        subwf   peak1,W
Sndm_C: btfsc   status,C
        goto    Sndm_D
        movf    adresh,W        ;if adresh:adresl > peak2:peak1
        movwf   peak2           ;save adresh:adresl to peak2:peak1
        bsf     status,RP0      ;bank 1
        movf    adresl,W
        bcf     status,RP0      ;bank 0
        movwf   peak1
Sndm_D: movf    adresh,W
        subwf   down2,W
        btfss   status,Z        ;if adresh <> down2
        goto    Sndm_E          ;skip to Sndm_E
        bsf     status,RP0      ;bank 1
        movf    adresl,W        ;else test adresl
        bcf     status,RP0      ;bank 0
        subwf   down1,W
Sndm_E: btfss   status,C
        goto    Sndm_F
        movf    adresh,W        ;if adresh:adresl < peak2:peak1
        movwf   down2           ;save adresh:adresl to down2:down1
        bsf     status,RP0      ;bank 1
        movf    adresl,W
        bcf     status,RP0      ;bank 0	
        movwf   down1
Sndm_F: decfsz  smpcnt,F        ;loop
        goto    Sndm_A
        return
;
Subtr:  movf    peak1,W         ;compute peak2:peak1 - cnst2:cnst1
        movwf   lat1            ;if peak2:peak1 > cnst2:cnst1,
        movf    peak2,W         ; Wreg will be set
        movwf   lat2            ;else Wreg will be cleared
        movf    cnst2,W
        subwf   lat2,F
        btfss   status,Z
        goto    Subtr2
        movf    cnst1,W
        subwf   lat1,F
Subtr2: btfsc   status,C
        retlw   0xFF
        retlw   0x00
;**************************************************************************
; end of PVK40 Example : Sound-level-meter
        end
