;**************************************************************************
;
;                           PVK40 Example : Sound-level-meter
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on E0 SEGM9 on S9
; 4) Switch on E1 SEGM10 on S9
; 5) Switch on A5 MIC on S9
; 6) Switch on C1 BARGRAPH on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P18f4520.INC>
; General Purpose Register definition :
smpcnt  equ     0x00            ;used in Sndm subroutine
peak1   equ     0x01            ;maximum, LSB
peak2   equ     0x02            ;maximum, MSB
down1   equ     0x03            ;minimum, LSB
down2   equ     0x04            ;minimum, MSB
cnst1   equ     0x05            ;constant, LSB
cnst2   equ     0x06            ;constant, MSB
bargr1  equ     0x07            ;bargraph data, LSB
bargr2  equ     0x08            ;bargraph data, MSB
cmpres  equ     0x09            ;Comp subroutine result
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  clrf    portc,a         ;clear portc latch
        setf    portd,a         ;set portd latch
        setf    porte,a         ;set porte latch
        movlw   B'11111101'     ;pin RC1 is output
        movwf   trisc,a
        clrf    trisd,a         ;portd pins are outputs
        movlw   B'00001100'     ;pins RE0,1 are outputs
        movwf   trise,a
        movlw   B'00010001'     ;enable A/D converter module
        movwf   adcon0,a
        movlw   B'00001010'     ;pin RA5 is analog input
        movwf   adcon1,a
        movlw   B'10001100'     ;A/D Conversion Clock = FOSC/4
        movwf   adcon2,a        ;A/D Acquisition Time = 8 / FOSC
        movlw   B'11000000'     ;prescaler 1:2 assigned to TMR0
        movwf   t0con,a         ;TMR0 period = 0.625 ms
        clrf    tmr0h,a
        clrf    tmr0l,a
        bcf     intcon,TMR0IF,a ;clear timer0 overflow flag
;
Main:   call    Sndm            ;measure maximum and minimum value
        movf    down1,w,a       ;peak2:peak1 - down2:down1 to peak2:peak1
        subwf   peak1,f,a
        movf    down2,w,a
        subwfb  peak2,f,a
        setf    bargr1,a
        setf    bargr2,a
        movlw   peak1           ;FSR0 points to peak register
        movwf   fsr0l,a
        clrf    fsr0h,a
        movlw   cnst1           ;FSR1 points to cnst register
        movwf   fsr1l,a
        clrf    fsr1h,a
;
        movlw   0x2D            ;cnst2:1 = 45
        movwf   cnst1,a	
        clrf    cnst2,a
        call    Comp            ;if peak2:1 > cnst
        btfsc   cmpres,0,a
        bcf     bargr1,0,a      ;switch on bargraph segment 1
;
        movlw   0x40            ;cnst2:1 = 64
        movwf   cnst1,a	
        call    Comp            ;if peak2:1 > cnst
        btfsc   cmpres,0,a
        bcf     bargr1,1,a      ;switch on bargraph segment 2
;
        movlw   0x5B            ;cnst2:1 = 91
        movwf   cnst1,a	
        call    Comp            ;if peak2:1 > cnst
        btfsc   cmpres,0,a
        bcf     bargr1,2,a      ;switch on bargraph segment 3
;
        movlw   0x80            ;cnst2:1 = 128
        movwf   cnst1,a	
        call    Comp            ;if peak2:1 > cnst
        btfsc   cmpres,0,a
        bcf     bargr1,3,a      ;switch on bargraph segment 4
;
        movlw   0xB5            ;cnst2:1 = 181
        movwf   cnst1,a	
        call    Comp            ;if peak2:1 > cnst
        btfsc   cmpres,0,a
        bcf     bargr1,4,a      ;switch on bargraph segment 5
;
        clrf    cnst1,a         ;cnst2:1 = 256	
        incf    cnst2,f,a
        call    Comp            ;if peak2:1 > cnst
        btfsc   cmpres,0,a
        bcf     bargr1,5,a      ;switch on bargraph segment 6
;
        movlw   0x39            ;cnst2:1 = 313
        movwf   cnst1,a
        call    Comp            ;if peak2:1 > cnst
        btfsc   cmpres,0,a
        bcf     bargr1,6,a      ;switch on bargraph segment 7
;
        clrf    cnst1,a         ;cnst2:1 = 512
        incf    cnst2,f,a
        call    Comp            ;if peak2:1 > cnst
        btfsc   cmpres,0,a
        bcf     bargr1,7,a      ;switch on bargraph segment 8
;
        movlw   0x80            ;cnst2:1 = 640
        movwf   cnst1,a	
        call    Comp            ;if peak2:1 > cnst
        btfsc   cmpres,0,a
        bcf     bargr2,0,a      ;switch on bargraph segment 9
;
        movlw   0xFA            ;cnst2:1 = 762
        movwf   cnst1,a
        call    Comp            ;if peak2:1 > cnst
        btfsc   cmpres,0,a
        bcf     bargr2,1,a      ;switch on bargraph segment 10
;
        movf    bargr2,W,a      ;write to bargraph
        movwf   porte,a
        movf    bargr1,W,a
        movwf   portd,a
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Sndm:   clrf    peak1,a         ;detect maximum and minimum value
        clrf    peak2,a
        setf    down1,a
        setf    down2,a
        clrf    smpcnt,a        ;256 samples
        movlw   adresl          ;FSR0 points to adres register
        movwf   fsr0l,a
        movlw   0x0F
        movwf   fsr0h,a
Sndm_A: bsf     adcon0,GO,a     ;starts A/D conversion
Sndm_B: btfsc   adcon0,GO,a     ;wait for A/D conversion to complete
        goto    Sndm_B
        btfss   intcon,TMR0IF,a ;wait for TMR0 overflow
        goto    Sndm_B
        bcf     intcon,TMR0IF,a ;clear timer0 overflow flag
;
        movlw   peak1           ;FSR1 points to peak register
        movwf   fsr1l,a
        clrf    fsr1h,a
        call    Comp            ;compare
        btfss   cmpres,0,a      ;if adresh:adresl > peak2:peak1
        bra     Sndm_C
        movff   adresh,peak2    ;save adresh:adresl to peak2:peak1
        movff   adresl,peak1
        bra     Sndm_D
;
Sndm_C: movlw   down1           ;FSR1 points to down register
        movwf   fsr1l,a
        call    Comp            ;compare
        btfss   cmpres,7,a      ;if adresh:adresl < down2:down1
        bra     Sndm_D
        movff   adresh,down2    ;save adresh:adresl to down2:down1
        movff   adresl,down1
Sndm_D: decfsz  smpcnt,F,a      ;loop
        goto    Sndm_A
        return
;
Comp:   movf    preinc0,w,a     ;compare two 16-bit register A,B
        cpfsgt  preinc1,a       ;A = *(FSR0,FSR0+1), B = *(FSR1,FSR1+1)
        bra     Comp_B          ;A < B => cmpres = B'11100000'
        movf    postdec0,f,a    ;A = B => cmpres = B'00011000'
        movf    postdec1,f,a    ;A > B => cmpres = B'00000111'
Comp_A: movlw   B'11100000'
        movwf   cmpres,a        ;A < B
        return
Comp_B: movf    postdec0,w,a
        cpfslt  postdec1,a
        bra     Comp_D
Comp_C: movlw   B'00000111'     ;A > B
        movwf   cmpres,a
        return
Comp_D: movf    indf0,w,a
        cpfseq  indf1,a
        bra     Comp_E
        movlw   B'00011000'     ;A = B
        movwf   cmpres,a
        return
Comp_E: cpfsgt  indf1,a
        goto    Comp_C
        goto    Comp_A
;**************************************************************************
; end of PVK40 Example : Sound-level-meter
        end
