;**************************************************************************
;
;                         PVK40 Example : UART - Tx
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; Serial port : 9600 Bd, 8 db, parity none, 1 stop bit, flow control none
;
; On your PVK40 board :
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on C6 UART_TX on S11
; 4) If USB selected, insert UMS1 module into socket
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f877a.INC>
; General Purpose Register definition :
pointer equ     0x20            ;points to characters in Table
latchw  equ     0x21            ;save Wreg
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  movlw   B'10000000'     ;serial port enabled, pin RC6 is
        movwf   rcsta           ; automaticaly configured as output
        bsf     status,RP0      ;bank 1
        movlw   0x14            ;9600 Bd
        movwf   spbrg
        movlw   B'00100110'     ;enable transmit
        movwf   txsta
        bcf     status,RP0      ;bank 0
;
        clrf    pointer
Main:   call    Table
        movwf   latchw          ;make advance
        movf    latchw,F
        btfsc   status,Z        ;repeat procedure until latchw = 0x00
        goto    Loop            ;last character in table was found
        movf    latchw,W
        call    Send            ;send character from table
        incf    pointer,F       ;take next character
        goto    Main
;
Loop:   goto    Loop            ;never-ending loop
;--------------------------------------------------------------------------
; Subroutines :
Send:   movwf   txreg           ;send data
        bsf     status,RP0      ;bank 1
Send2:  btfss   txsta,TRMT
        goto    Send2
        bcf     status,RP0      ;bank 0
        return
;
Table:  movf    pointer,W       ;table of legend
        andlw   B'00001111'     ;overflow prevention
        addwf   pcl,f
        retlw   'H'
        retlw   'e'
        retlw   'l'
        retlw   'l'
        retlw   'o'
        retlw   ','
        retlw   ' '
        retlw   'W'
        retlw   'o'
        retlw   'r'
        retlw   'l'
        retlw   'd'
        retlw   '!'
        retlw   0x0D            ;new line
        retlw   0x0A
        retlw   0x00
        retlw   0x00            ;prohibited zone
        retlw   0x00            ;prohibited zone
;**************************************************************************
; end of PVK40 Example : UART - Tx
        end
