;**************************************************************************
;
;                         PVK40 Example : UART - Rx
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; Serial port : 9600 Bd, 8 db, parity none, 1 stop bit, flow control none
;
; On your PVK40 board :
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on C6 UART_RX on S11
; 4) Switch on B0,1,2,4 DISP on S11
; 5) Select COM or USB on JP4
; 6) If USB selected, insert UMS1 module into socket
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f877a.INC>
; General Purpose Register definition :
rcdata  equ     0x20            ;received data
bckwreg equ     0x21            ;working register backup during interrupt
; Bits definition :
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x04            ;interrupt vector
Interr: movwf   bckwreg         ;save wreg during interrupt
        movf    rcreg,W         ;read received data
        movwf   rcdata          ;RCIF will be automaticaly cleared
        movf    bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  bsf     status,RP0      ;bank 1
        movlw   B'11111100'     ;pins RB0,1 are outputs
        movwf   trisb
        clrf    trisd           ;portd pins are outputs
        movlw   0x14            ;9600 Bd
        movwf   spbrg
        movlw   B'00000110'     ;disable transmit
        movwf   txsta
        movlw   B'00100000'     ;unmask interrupt from UART Rx
        movwf   pie1
        bcf     status,RP0      ;bank 0
        movlw   B'10010000'     ;serial port enabled
        movwf   rcsta
        movlw   B'01000000'     ;unmask peripheral interrupt
        movwf   intcon
        clrf    rcdata
        movf    rcreg,W         ;clear RCIF
        bsf     intcon,GIE      ;enable interrupt
;
Main:   movf    rcdata,W        ;first digit to portd
        andlw   0x0F
        call    Table
        movwf   portd
        bcf     DCOM1           ;turn on display 1
        nop
        nop
        nop
        bsf     DCOM1           ;turn off display 1
        swapf   rcdata,W        ;second digit to portd
        andlw   0x0F
        call    Table
        movwf   portd
        bcf     DCOM2           ;turn on display 2
        nop
        nop
        nop
        bsf     DCOM2           ;turn off display 2
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  andlw   0x0F            ;Display segments table
        addwf   pcl,f
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'10001000'     ;A
        retlw   B'10000011'     ;B
        retlw   B'11000110'     ;C
        retlw   B'10100001'     ;D
        retlw   B'10000110'     ;E
        retlw   B'10001110'     ;F
;**************************************************************************
; end of PVK40 Example : UART - Rx
        end
