;**************************************************************************
;
;                         PVK40 Example : UART - Rx
;
; Processor : PIC18F4520
; Clock : 13.1072 MHz, crystal 3.2768 MHz with x4 PLL used
;
; Serial port : 115200 Bd,8 db,parity none,1 stop bit,flow control none
;
; On your PVK40 board :
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on C6 UART_RX on S11
; 4) Switch on B0,1,2,4 DISP on S11
; 5) Select COM or USB on JP4
; 6) If USB selected, insert UMS1 module into socket
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x06  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P18f4520.INC>
; General Purpose Register definition :
rcdata  equ     0x00            ;received data
; Bits definition :
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;
        org     0x08            ;interrupt vector
Interr: movf    rcreg,W,a       ;read received data
        movwf   rcdata,a        ;RCIF will be automaticaly cleared
        retfie  fast            ;fast return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   B'11111100'     ;pins RB0,1 are outputs
        movwf   trisb,a
        clrf    trisd,a         ;portd pins are outputs
        movlw   B'10010000'     ;serial port enabled
        movwf   rcsta,a
        movlw   0x06            ;115200 Bd
        movwf   spbrg,a
        movlw   B'01000000'
        movwf   baudcon,a
        movlw   B'00000110'     ;disable transmit
        movwf   txsta,a
        movlw   B'00100000'     ;unmask interrupt from UART Rx
        movwf   pie1,a
        bcf     rcon,IPEN,a     ;disable priority interrupt
        movlw   B'01000000'     ;unmask peripheral interrupt
        movwf   intcon,a
        clrf    rcdata,a
        movf    rcreg,W,a       ;clear RCIF
        bsf     intcon,GIE,a    ;enable interrupt
;
Main:   movf    rcdata,W,a      ;first digit to portd
        andlw   0x0F
        call    Table
        movwf   portd,a
        bcf     DCOM1,a         ;turn on display 1
        nop
        nop
        nop
        bsf     DCOM1,a         ;turn off display 1
        swapf   rcdata,W,a      ;second digit to portd
        andlw   0x0F
        call    Table
        movwf   portd,a
        bcf     DCOM2,a         ;turn on display 2
        nop
        nop
        nop
        bsf     DCOM2,a         ;turn off display 2
        goto    Main            ;closes the loop
;--------------------------------------------------------------------------
; Subroutines :
Table:  rlncf   wreg,f,a        ;Display segments table
        addwf   pcl,f,a         ;wreg * 2 -> pcl
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'10001000'     ;A
        retlw   B'10000011'     ;B
        retlw   B'11000110'     ;C
        retlw   B'10100001'     ;D
        retlw   B'10000110'     ;E
        retlw   B'10001110'     ;F
;**************************************************************************
; end of PVK40 Example : UART - Rx
        end
