;**************************************************************************
;
;                           PVK40 Example : Alarm clock
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on B0,1,2,4 DISP on S11
; 4) Switch on B3 LED on S11
; 5) Switch on C2 BUZZER on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P18f4520.INC>
; General Purpose Register definition :
cnt1    equ     0x00            ;used in Wait subroutine
cnt2    equ     0x01            ;used in Wait subroutine
bpcnt   equ     0x02            ;beep counting
discyc  equ     0x03            ;display cycle
btns    equ     0x04            ;pushbuttons state, 0 - 4
ontime  equ     0x05            ;bit 0 = alarm flag
thrsh   equ     0x06            ;real time, 10 * hours, 0 - 2
thrsl   equ     0x07            ;real time, hours, 0 - 9 / 0 - 3
tminh   equ     0x08            ;real time, 10 * minutes 0 - 5
tminl   equ     0x09            ;real time, minutes 0 - 9
tsec    equ     0x0A            ;real time, 1/2 seconds, 0 - 119
tmsec   equ     0x0B            ;real time, 1/400 seconds, 0 - 199
ahrsh   equ     0x0C            ;alarm time, 10 * hours, 0 - 2
ahrsl   equ     0x0D            ;alarm time, hours, 0 - 9 / 0 - 3
aminh   equ     0x0E            ;alarm time, 10 * minutes, 0 - 5
aminl   equ     0x0F            ;alarm time, minutes, 0 - 9
; Bits definition :
#define DCOM1   portb,0         ;display common anode 1
#define DCOM2   portb,1         ;display common anode 2
#define DCOM3   portb,2         ;display common anode 3
#define DCOM4   portb,4         ;display common anode 4
#define LED     portb,3         ;LED is on the RB3 pin
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;--------------------------------------------------------------------------
        org     0x08            ;interrupt vector
Interr: bcf     intcon,TMR0IF,a ;clear flag
        incf    tmsec,F,a       ;increment time
        movlw   0xC8            ;1/400 second
        subwf   tmsec,W,a
        bnz     $+6
        clrf    tmsec,a
        incf    tsec,F,a
        movlw   0x78            ;1/2 second
        subwf   tsec,W,a
        bnz     $+6
        clrf    tsec,a
        incf    tminl,F,a
        movlw   0x0A            ;minutes
        subwf   tminl,W,a
        bnz     $+6
        clrf    tminl,a
        incf    tminh,F,a
        movlw   0x06            ;10 * minutes
        subwf   tminh,W,a
        bnz     $+6
        clrf    tminh,a
        incf    thrsl,F,a
        movlw   0x0A            ;hours
        btfsc   thrsh,1,a       ;if thrsh = 2 then thrsl = 0 .. 3
        movlw   0x04
        subwf   thrsl,W,a
        bnz     $+6
        clrf    thrsl,a
        incf    thrsh,F,a
        movlw   0x03            ;10 * hours
        cpfslt  thrsh,a
        clrf    thrsh,a
;
        tstfsz  tmsec,a         ;test if real time = alarm time
        bra     Tfd
        tstfsz  tsec,a          ;tmsec = 0
        bra     Tfd
        movf    tminl,W,a       ;tsec = 0
        cpfseq  aminl,a         ;test if tminl = aminl
        bra     Tfd
        movf    tminh,W,a       ;test if tminh = aminh
        cpfseq  aminh,a
        bra     Tfd
        movf    thrsl,W,a       ;test if thrsl = ahrsl
        cpfseq  ahrsl,a
        bra     Tfd
        movf    thrsh,W,a       ;test if thrsh = ahrsh
        cpfseq  ahrsh,a
        bra     Tfd
        bsf     ontime,0,a      ;if real_time=alarm_time then set flag
;
Tfd:    bsf     DCOM1,a         ;turn off display 1
        bsf     DCOM2,a         ;turn off display 2
        bsf     DCOM3,a         ;turn off display 3
        bsf     DCOM4,a         ;turn off display 4
;
        incf    discyc,F,a      ;increment discyc
        movlw   0x05
        cpfslt  discyc,a
        clrf    discyc,a
        clrf    pclath,a
        rlncf   discyc,w,a
        addwf   pcl,f,a
        bra     Cyc0
        bra     Cyc1
        bra     Cyc2
        bra     Cyc3
        bra     Cyc4
        goto    Start           ;error -> reset
        goto    Start           ;error -> reset
        goto    Start           ;error -> reset
;
Cyc0:   movf    portd,W,a       ;read portd
        movwf   btns,a
        movf    thrsh,W,a       ;real time / preset time
        btfsc   LED,a
        movf    ahrsh,W,a
        call    Table
        movwf   portd,a
        clrf    trisd,a         ;portd pins are outputs
        bcf     DCOM4,a         ;turn on display 4
        retfie  fast            ;fast return and set GIE
;
Cyc1:   movf    thrsl,W,a       ;real time / preset time
        btfsc   LED,a
        movf    ahrsl,W,a
        call    Table
        btfss   tsec,0,a        ;blinking dot
        andlw   B'01111111'
        btfsc   LED,a
        andlw   B'01111111'
        movwf   portd,a
        bcf     DCOM3,a         ;turn on display 3
        retfie  fast            ;fast return and set GIE
;
Cyc2:   movf    tminh,W,a       ;real time / preset time
        btfsc   LED,a
        movf    aminh,W,a
        call    Table
        movwf   portd,a
        bcf     DCOM2,a         ;turn on display 2
        retfie  fast            ;fast return and set GIE
;
Cyc3:   movf    tminl,W,a       ;real time / preset time
        btfsc   LED,a
        movf    aminl,W,a
        call    Table
        movwf   portd,a
        bcf     DCOM1,a         ;turn on display 1
        retfie  fast            ;fast return and set GIE
;
Cyc4:   setf    trisd,a         ;portd pins are inputs
        retfie  fast            ;fast return and set GIE
;--------------------------------------------------------------------------
        org     0x0100
; Subroutines :
Table:  rlncf   wreg,f,a        ;Display segments table
        andlw   B'00011110'
        bsf     pclath,0,a
        addwf   pcl,f,a         ;wreg * 2 -> pcl
        retlw   B'11000000'     ;0
        retlw   B'11111001'     ;1
        retlw   B'10100100'     ;2
        retlw   B'10110000'     ;3
        retlw   B'10011001'     ;4
        retlw   B'10010010'     ;5
        retlw   B'10000010'     ;6
        retlw   B'11111000'     ;7
        retlw   B'10000000'     ;8
        retlw   B'10010000'     ;9
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
        retlw   B'11111111'     ;display off
;
Wait:   movlw   0x60            ;this subroutine wait 74021 cycles
        movwf   cnt1,a          ;(90ms)
Wait_B: clrf    cnt2,a
Wait_C: decfsz  cnt2,f,a        ;decrement cnt2
        goto    Wait_C
        decfsz  cnt1,f,a        ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;--------------------------------------------------------------------------
Start:  setf    portd,a         ;set portd
        movlw   B'00010111'
        movwf   portb,a         ;set RB0,1,2,4
        movlw   B'11100000'     ;pins RB0,1,2,3,4 are outputs
        movwf   trisb,a
        movlw   B'11111011'     ;pin RC2 is output
        movwf   trisc,a
        clrf    ccpr1h,a        ;PWM duty cycle = 0%
        clrf    ccpr1l,a
        movlw   B'00001100'     ;CCP1 mode = PWM
        movwf   ccp1con,a
        clrf    tmr2,a
        movlw   B'00000100'     ;timer2 on, prescaler 1:1
        movwf   t2con,a
        movlw   B'11111111'     ;PWM period
        movwf   pr2,a
;
        clrf    thrsh,a         ;start time 00:00:00.000
        clrf    thrsl,a
        clrf    tminh,a
        clrf    tminl,a
        clrf    tsec,a
        clrf    tmsec,a
        movlw   0x01            ;alarm time 12:00
        movwf   ahrsh,a
        movlw   0x02
        movwf   ahrsl,a
        clrf    aminh,a
        clrf    aminl,a
        clrf    discyc,a
        setf    btns,a
        clrf    ontime,a
;
        movlw   B'11000010'     ;prescaler assigned to TMR0
        movwf   t0con,a         ;TMR0 overflow period = 2.5ms
        clrf    tmr0h,a
        clrf    tmr0l,a
        bcf     rcon,IPEN,a     ;disable priority interrupt
        movlw   B'00100000'     ;unmask interrupt from TMR0
        movwf   intcon,a
        bsf     intcon,GIE,a    ;enable global interrupt
;
Main:   btfss   btns,0,a        ;test pushbutton 0
        goto    Btn0
        btfss   btns,4,a        ;test pushbutton 4
        goto    Btn4
        btfss   btns,5,a        ;test pushbutton 5
        goto    Btn5
        btfss   btns,6,a        ;test pushbutton 6
        goto    Btn6
        btfss   btns,7,a        ;test pushbutton 7
        goto    Btn7
        btfss   ontime,0,a      ;test ontime flag
        goto    Main            ;closes the loop
;
        movlw   0x4B            ;Alarm
        movwf   bpcnt,a
        bcf     ontime,0,a
Alarm:  movlw   0x80            ;PWM on
        movwf   ccpr1l,a
        call    Wait
        clrf    ccpr1l,a        ;PWM off
        call    Wait
        call    Wait
        incf    btns,W,a        ;test pushbuttons
        btfss   status,Z,a
        bra     Alarm2
        decfsz  bpcnt,F,a
        goto    Alarm
Alarm2: incf    btns,W,a        ;test pushbuttons
        btfss   status,Z,a      ;wait for release
        goto    Alarm2
        goto    Main
;--------------------------------------------------------------------------
Btn0:   btg     LED,a           ;change LED state
        call    Wait
Btn0_A: btfss   btns,0,a        ;wait for release
        goto    Btn0_A
        call    Wait
        goto    Main
;
Btn4:   bcf     intcon,GIE,a    ;disable interrupt
        btfsc   LED,a           ;real time / alarm time
        bra     Btn4_A
        incf    thrsl,F,a       ;increment real time
        movlw   0x0A            ;hours
        btfsc   thrsh,1,a       ;if thrsh = 2 then thrsl = 0 .. 3
        movlw   0x04
        subwf   thrsl,W,a
        bnz     $+6
        clrf    thrsl,a
        incf    thrsh,F,a
        movlw   0x03            ;10 * hours
        cpfslt  thrsh,a
        clrf    thrsh,a
        bra     Btn4_B
;
Btn4_A: incf    ahrsl,F,a       ;increment alarm time
        movlw   0x0A            ;hours
        btfsc   ahrsh,1,a       ;if ahrsh = 2 then ahrsl = 0 .. 3
        movlw   0x04
        subwf   ahrsl,W,a
        bnz     $+6
        clrf    ahrsl,a
        incf    ahrsh,F,a
        movlw   0x03            ;10 * hours
        cpfslt  ahrsh,a
        clrf    ahrsh,a
Btn4_B: bsf     intcon,GIE,a    ;enable interrupt
        incf    btns,W,a        ;test pushbuttons
        bnz     Btn4_B          ;wait for release
        goto    Main
;
Btn5:   bcf     intcon,GIE,a    ;disable interrupt
        btfsc   LED,a           ;real time / alarm time
        bra     Btn5_A
        decf    thrsl,f,a       ;decrement real time - hours
        bc      Btn5_B
        movlw   0x09
        movwf   thrsl,a
        decf    thrsh,f,a       ;10 * hours
        bc      Btn5_B
        movlw   0x02
        movwf   thrsh,a
        movlw   0x03
        movwf   thrsl,a
        bra     Btn5_B
;
Btn5_A: decf    ahrsl,f,a       ;decrement alarm time - hours
        bc      Btn5_B
        movlw   0x09
        movwf   ahrsl,a
        decf    ahrsh,f,a       ;10 * hours
        bc      Btn5_B
        movlw   0x02
        movwf   ahrsh,a
        movlw   0x03
        movwf   ahrsl,a
Btn5_B: bsf     intcon,GIE,a    ;enable interrupt
        incf    btns,W,a        ;test pushbuttons
        bnz     Btn5_B          ;wait for release
        goto    Main
;
Btn6:   bcf     intcon,GIE,a    ;disable interrupt
        btfsc   LED,a           ;real time / alarm time
        bra     Btn6_A
        incf    tminl,F,a       ;increment real time
        movlw   0x0A            ;minutes
        subwf   tminl,W,a
        bnz     $+6
        clrf    tminl,a
        incf    tminh,F,a
        movlw   0x06            ;10 * minutes
        cpfslt  tminh,a
        clrf    tminh,a
        bra     Btn6_B
;
Btn6_A: incf    aminl,F,a       ;increment alarm time
        movlw   0x0A            ;minutes
        subwf   aminl,W,a
        bnz     $+6
        clrf    aminl,a
        incf    aminh,F,a
        movlw   0x06            ;10 * minutes
        cpfslt  aminh,a
        clrf    aminh,a
Btn6_B: bsf     intcon,GIE,a    ;enable interrupt
        incf    btns,W,a        ;test pushbuttons
        bnz     Btn6_B          ;wait for release
        goto    Main
;
Btn7:   bcf     intcon,GIE,a    ;disable interrupt
        btfsc   LED,a           ;real time / alarm time
        bra     Btn7_A
        decf    tminl,f,a       ;decrement real time - minutes
        bc      Btn7_B
        movlw   0x09
        movwf   tminl,a
        decf    tminh,f,a       ;10 * minutes
        bc      Btn7_B
        movlw   0x05
        movwf   tminh,a
        bra     Btn7_B
Btn7_A: decf    aminl,f,a       ;decrement alarm time - minutes
        bc      Btn7_B
        movlw   0x09
        movwf   aminl,a
        decf    aminh,f,a       ;10 * minutes
        bc      Btn7_B
        movlw   0x05
        movwf   aminh,a
Btn7_B: bsf     intcon,GIE,a    ;enable interrupt
        incf    btns,W,a        ;test pushbuttons
        bnz     Btn7_B          ;wait for release
        goto    Main
;**************************************************************************
; end of PVK40 Example : Alarm clock
        end
