;**************************************************************************
;
;                           PVK40 Example : Security alarm
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; On your PVK40 board set DIP switches according to following way:
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on TEMP on S9
; 4) Switch on PHOTO on S9
; 5) Switch on MIC on S9
; 6) Switch on B3 LED on S11
; 7) Switch on C2 BUZZER on S11
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
; EEPROM data :
        org     0x2100
        de      0x00,0x01,0x02,0x03      ;code (0123)
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f877a.INC>
; General Purpose Register definition :
cnt1    equ     0x20            ;used in Wait subroutine
cnt2    equ     0x21            ;used in Wait subroutine
intcyc  equ     0x22            ;interrupt cycle
modcnt  equ     0x23            ;used in Mod subroutine
moddat  equ     0x24            ;used in Mod subroutine
scnt    equ     0x25            ;used in Siren subroutine
bcnt    equ     0x26            ;LED blink counting
codcnt  equ     0x27            ;code digit counting
codd1   equ     0x28            ;code, digit 1
codd2   equ     0x29            ;code, digit 2
codd3   equ     0x2A            ;code, digit 3
codd4   equ     0x2B            ;code, digit 4
ecodd1  equ     0x2C            ;entered code, digit 1
ecodd2  equ     0x2D            ;entered code, digit 2
ecodd3  equ     0x2E            ;entered code, digit 3
ecodd4  equ     0x2F            ;entered code, digit 4
templ   equ     0x30            ;temperature, LSB
temph   equ     0x31            ;temperature, MSB
photol  equ     0x32            ;photosensor value, LSB
photoh  equ     0x33            ;photosensor value, MSB
micl    equ     0x34            ;sound level, LSB
mich    equ     0x35            ;sound level, MSB
atempl  equ     0x36            ;actual temperature, LSB
atemph  equ     0x37            ;actual temperature, MSB
aphotol equ     0x38            ;actual photosensor value, LSB
aphotoh equ     0x39            ;actual photosensor value, MSB
amicl   equ     0x3A            ;actual sound level, LSB
amich   equ     0x3B            ;actual sound level, MSB
intcyct equ     0x3C            ;temporary interrupt cycle
peak1   equ     0x3D            ;microphone maximum, LSB
peak2   equ     0x3E            ;microphone maximum, MSB
down1   equ     0x3F            ;microphone minimum, LSB
down2   equ     0x40            ;microphone minimum, MSB
cmpoff  equ     0x41            ;used in Cmp subroutine, offset
cmptol  equ     0x42            ;used in Cmp subroutine, tolerance
cmpal   equ     0x43            ;used in Cmp subroutine
cmpah   equ     0x44            ;used in Cmp subroutine
cmpbl   equ     0x45            ;used in Cmp subroutine
cmpbh   equ     0x46            ;used in Cmp subroutine
ilbl    equ     0x47            ;LED blink counting
bcksta  equ     0x70            ;status register backup during interrupt
bckwreg equ     0x71            ;working register backup during interrupt
bckfsr  equ     0x72            ;fsr register backup during interrupt
; Bits definition :
#define LED     portb,3         ;LED is on the RB3 pin
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;--------------------------------------------------------------------------
        org     0x04            ;interrupt vector
Inter:  movwf   bckwreg         ;save wreg during interrupt
        swapf   status,w        ;save status during interrupt
        movwf   bcksta
        movf    fsr,W           ;save fsr during interrupt
        movwf   bckfsr
        bcf     status,RP0      ;bank 0
        bcf     intcon,T0IF     ;clear flag
;
        movf    intcyc,W        ;test intcyc
        andlw   B'01111111'
        movwf   intcyct
        sublw   0x05
        movlw   0x06
        btfss   status,C        ;if intcyct > 5
        movwf   intcyct         ;then intcyct = 6
        movf    intcyc,W        ;if intcyc = 127
        andlw   B'01111111'
        xorlw   B'01111111'
        movlw   0x07
        btfsc   status,Z        ;then intcyct = 7
        movwf   intcyct
;
        movf    intcyct,W       ;skip to corresponding Cycx
        addwf   pcl,f
        goto    Cyc0
        goto    Cyc1
        goto    Cyc2
        goto    Cyc3
        goto    Cyc4
        goto    Cyc5
        goto    Cyc6
        goto    Cyc7
;
Cyc0:   movlw   B'01000001'     ;Cycle 0 - select temperature sensor
        movwf   adcon0
        goto    Cycfin
;
Cyc1:   bsf     adcon0,GO       ;Cycle 1 - start conversion
        goto    Cycfin
;
Cyc2:   btfsc   adcon0,GO       ;Cycle 2 - save temperature
        goto    Cyc2            ;and select photosensor
        movf    adresh,W
        movwf   atemph
        bsf     status,RP0      ;bank 1
        movf    adresl,W
        bcf     status,RP0      ;bank 0
        movwf   atempl
        movlw   B'01001001'
        movwf   adcon0
        goto    Cycfin
;
Cyc3:   bsf     adcon0,GO       ;Cycle 3 - start conversion
        goto    Cycfin
;
Cyc4:   btfsc   adcon0,GO       ;Cycle 4 - save photosensor value
        goto    Cyc4            ;and select microphone
        movf    adresh,W
        movwf   aphotoh
        bsf     status,RP0      ;bank 1
        movf    adresl,W
        bcf     status,RP0      ;bank 0
        movwf   aphotol
        movlw   B'01100001'
        movwf   adcon0
        goto    Cycfin
;
Cyc5:   bsf     adcon0,GO       ;Cycle 5 - start conversion
        clrf    peak1
        clrf    peak2
        movlw   0xFF
        movwf   down1
        movwf   down2
        goto    Cycfin
;
Cyc6:   btfsc   adcon0,GO       ;Cycle 6..126 - save mic. min/max value
        goto    Cyc6            ;and start new conversion
        movf    adresh,W
        subwf   peak2,W
        btfss   status,Z        ;if adresh <> peak2
        goto    Cyc6_A          ;skip Cyc6_A
        bsf     status,RP0      ;bank 1
        movf    adresl,W        ;else test adresl
        bcf     status,RP0      ;bank 0
        subwf   peak1,W
Cyc6_A: btfsc   status,C
        goto    Cyc6_B
        movf    adresh,W        ;if adresh:adresl > peak2:peak1
        movwf   peak2           ;save adresh:adresl to peak2:peak1
        bsf     status,RP0      ;bank 1
        movf    adresl,W
        bcf     status,RP0      ;bank 0
        movwf   peak1
Cyc6_B: movf    adresh,W
        subwf   down2,W
        btfss   status,Z        ;if adresh <> down2
        goto    Cyc6_C          ;skip to Cyc6_C
        bsf     status,RP0      ;bank 1
        movf    adresl,W        ;else test adresl
        bcf     status,RP0      ;bank 0
        subwf   down1,W
Cyc6_C: btfss   status,C
        goto    Cyc6_D
        movf    adresh,W        ;if adresh:adresl < peak2:peak1
        movwf   down2           ;save adresh:adresl to down2:down1
        bsf     status,RP0      ;bank 1
        movf    adresl,W
        bcf     status,RP0      ;bank 0	
        movwf   down1
Cyc6_D: bsf     adcon0,GO
        goto    Cycfin
;
Cyc7:   movf    down1,W         ;Cycle 7 - save sound level
        subwf   peak1,W         ;peak2:peak1 - down2:down1 to amich:l
        movwf   amicl
        btfss   status,C
        decf    peak2,F
        movf    down2,W
        subwf   peak2,W
        movwf   amich
        goto    Cycfin
;
Cycfin: incf    intcyc,F
        movlw   0x80
        subwf   intcyc,W        ;if intcyc = 128
        btfsc   status,Z
        goto    Svcon           ;then save primary values
        movf    intcyc,F
        btfss   status,Z        ;if intcyc <> 0
        goto    Intret          ;then return
        movlw   0x80            ;if intcyc = 0 then intcyc = 128
        movwf   intcyc
;
        movf    ilbl,F          ;test if ilbl = 0
        btfss   status,Z
        goto    Nlbl
        movlw   0x06            ;compare all values
        movwf   cmpoff          ;templ - atempl offset
        movlw   0x01
        movwf   cmptol          ;tolerance
        movlw   templ           ;temp register address
        call    Comp
        movwf   cmptol
        btfsc   cmptol,0        ;test result
        goto    Slbl
        movlw   0x20
        movwf   cmptol          ;tolerance
        movlw   photol          ;photo register address
        call    Comp
        movwf   cmptol
        btfsc   cmptol,0        ;test result
        goto    Slbl
        movlw   0x35
        movwf   cmptol          ;tolerance
        movlw   micl            ;mic register address
        call    Comp
        movwf   cmptol
        btfsc   cmptol,0        ;test result
        goto    Slbl
        goto    Intret          ;values OK, return
;
Slbl:   movlw   0x01            ;event on sensor
        movwf   ilbl            ;start LED blinking
        goto    Intret
;
Nlbl:   incf    ilbl,F          ;increment time
        btfss   LED             ;invert LED state
        goto    Nlbl_A
        bcf     LED
        goto    Nlbl_B
Nlbl_A: bsf     LED
Nlbl_B: movlw   0x20            ;max. time
        xorwf   ilbl,W
        btfss   status,Z
        goto    Intret
        bsf     LED
        call    Siren           ;time overflow, start siren
        goto    Enin
;
Svcon:  movf    atempl,W        ;save primary values
        movwf   templ
        movf    atemph,W
        movwf   temph
        movf    aphotol,W
        movwf   photol
        movf    aphotoh,W
        movwf   photoh
        movf    amicl,W
        movwf   micl
        movf    amich,W
        movwf   mich
        clrf    ilbl            ;clear ilbl = no event
;
Intret: movf    bckfsr,W        ;restore fsr
        movwf   fsr
        swapf   bcksta,w        ;restore status register
        movwf   status
        swapf   bckwreg,f
        swapf   bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
; Subroutines :
Rdee:   bsf     status,RP1      ;read from internal EEPROM; bank 2
        movwf   eeadr           ;Write address to read from
        bsf     status,RP0      ;bank 3
        bcf     eecon1,EEPGD    ;point to Data memory
        bsf     eecon1,RD       ;start read operation
        bcf     status,RP0      ;bank 2
        movf    eedata,W        ;W = EEDATA
        bcf     status,RP1      ;bank 0
        return
;
Mod:    andlw   0x07            ;modify 0-7 to pushbutton code
        addlw   0x01
        movwf   modcnt
        movlw   0xFF
        movwf   moddat          ;set moddat
        bcf     status,C        ;clear C
Mod2:   rlf     moddat,F        ;rotate
        decfsz  modcnt,F
        goto    Mod2
        movf    moddat,W
        return
;
Beep:   bsf     status,RP0      ;bank 1
        movlw   B'01000000'     ;PWM period
        movwf   pr2
        bcf     status,RP0      ;bank 0
        movlw   0x02            ;PWM on
        movwf   ccpr1l
        call    Wait
        clrf    ccpr1l          ;PWM off
        return
;
Siren:  movlw   0x20            ;Siren
        movwf   scnt
Siren2: bsf     status,RP0      ;bank 1
        movlw   B'01000000'     ;PWM period
        movwf   pr2
        bcf     status,RP0      ;bank 0
        movlw   B'00100000'     ;PWM on
        movwf   ccpr1l
        call    Wait
        call    Wait
        call    Wait
        bsf     status,RP0      ;bank 1
        movlw   B'10000000'     ;PWM period
        movwf   pr2
        bcf     status,RP0      ;bank 0
        movlw   B'01000000'
        movwf   ccpr1l
        call    Wait
        call    Wait
        call    Wait
        decfsz  scnt,F
        goto    Siren2
        clrf    ccpr1l          ;PWM off
        return
;
Wait:   movlw   0x60            ;this subroutine wait 74021 cycles
        movwf   cnt1            ;(90ms)
Wait_B: clrf    cnt2
Wait_C: decfsz  cnt2,f          ;decrement cnt2
        goto    Wait_C
        decfsz  cnt1,f          ;decrement cnt1
        goto    Wait_B
        return                  ;if cnt1=0 then return
;
Keyin:  movlw   ecodd1          ;read 4 - digit code
        movwf   fsr
        clrf    codcnt
Keyin2: incf    portd,W         ;test pushbuttons
        btfsc   status,Z        ;wait for press pushbutton
        goto    Keyin2
        addlw   0xFF            ;subtract 1
        movwf   indf            ;save pushbuttons state
        call    Beep
Keyin3: incf    portd,W         ;test pushbuttons
        btfss   status,Z        ;wait for release
        goto    Keyin3
        call    Wait
        incf    fsr,F
        incf    codcnt,F
        btfss   codcnt,2
        goto    Keyin2          ;repeat 4x
;
        movf    codd1,W         ;compare codd4:1 with ecodd4:1
        xorwf   ecodd1,W
        btfss   status,Z
        retlw   0xFF
        movf    codd2,W
        xorwf   ecodd2,W
        btfss   status,Z
        retlw   0xFF
        movf    codd3,W
        xorwf   ecodd3,W
        btfss   status,Z
        retlw   0xFF
        movf    codd4,W
        xorwf   ecodd4,W
        btfss   status,Z
        retlw   0xFF
        retlw   0x00            ;code OK
;
Comp:   movwf   fsr             ;compare 16-bit number *Wreg +/-cmptol
        movf    indf,W          ;with 16-bit number *(Wreg + cmpoff)
        movwf   cmpal           ;copy indf to cmpah:l
        incf    fsr,F
        movf    indf,W
        movwf   cmpah
;
        movf    cmptol,W        ;cmpah:l = cmpah:l - tolerance
        subwf   cmpal,F
        btfsc   status,C
        goto    Comp2
        movlw   0xFF            ;subtract 1
        addwf   cmpah,F
        btfsc   status,C        ;if cmpah:l < tolerance
        goto    Comp2
        clrf    cmpal           ;then cmpah:l = 0
        clrf    cmpah
;
Comp2:  decf    fsr,W           ;copy indf to cmpbh:l
        addwf   cmpoff,W
        movwf   fsr
        movf    indf,W
        movwf   cmpbl
        incf    fsr,F
        movf    indf,W
        movwf   cmpbh
;
        movf    cmpah,W         ;compare cmpah:l with cmpbh:l
        subwf   cmpbh,W
        btfss   status,C
        retlw   0xFF            ;nonequal
        btfss   status,Z
        goto    Comp3
        movf    cmpal,W
        subwf   cmpbl,W
        btfss   status,C
        retlw   0xFF            ;nonequal
;
Comp3:  decf    fsr,F           ;copy indf to cmpah:l
        movf    cmpoff,W
        subwf   fsr,F
        movf    indf,W
        movwf   cmpal
        incf    fsr,F
        movf    indf,W
        movwf   cmpah
;
        movf    cmptol,W        ;cmpah:l = cmpah:l + tolerance
        addwf   cmpal,F
        btfss   status,C
        goto    Comp4
        incf    cmpah,F
        btfss   status,Z        ;if cmpah:l + tolerance > 0xFFFF
        goto    Comp4
        movlw   0xFF            ;then cmpah:l = 0xFFFF
        movwf   cmpal
        movwf   cmpah
;
Comp4:  movf    cmpbh,W         ;compare cmpah:l with cmpbh:l
        subwf   cmpah,W
        btfss   status,C
        retlw   0xFF            ;nonequal
        btfss   status,Z
        retlw   0x00            ;in tolerance
        movf    cmpbl,W
        subwf   cmpal,W
        btfss   status,C
        retlw   0xFF            ;nonequal
        retlw   0x00            ;in tolerance
;--------------------------------------------------------------------------
Start:  movlw   B'00000000'
        movwf   portb           ;clear RB3
        clrf    ccpr1h          ;PWM duty cycle = 0%
        clrf    ccpr1l
        movlw   B'00001111'     ;CCP1 mode = PWM
        movwf   ccp1con
        clrf    tmr2
        movlw   B'00000101'     ;timer2 on, prescaler 1:4
        movwf   t2con
        bsf     status,RP0      ;bank 1
        movlw   B'10000000'     ;porta pins are analog inputs
        movwf   adcon1
        movlw   B'11110111'     ;pin RB3 is output
        movwf   trisb
        movlw   B'11111011'     ;pin RC2 is output
        movwf   trisc
        movlw   B'11010001'     ;prescaler assigned to TMR0
        movwf   option_reg      ;TMR0 period = 1.25 ms
        bcf     status,RP0      ;bank 0
        movlw   B'01000001'     ;enable A/D converter module
        movwf   adcon0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
;
        movlw   codd4           ;read code from eeprom
        movwf   fsr
        movlw   0x04
        movwf   codcnt
Codrd:  decf    codcnt,W
        call    Rdee            ;read eeprom
        call    Mod             ;modify to pushbutton code
        movwf   indf
        decf    fsr,F
        decfsz  codcnt,F        ;4x
        goto    Codrd
;
Inact:  bcf     LED             ;turn off the LED
        call    Keyin           ;read 4 - digit code
        addlw   0x01
        btfss   status,Z        ;test result
        goto    Blled           ;code OK
        call    Siren           ;wrong code
        goto    Inact
;
Blled:  movlw   0x40            ;LED blink
        movwf   bcnt
Blled2: bcf     LED
        call    Wait
        bsf     LED
        call    Wait
        decfsz  bcnt,F
        goto    Blled2
;
Enin:   call    Wait
        bcf     intcon,T0IF     ;clear TMR0 flag
        clrf    intcyc          ;start with cycle 0
        bsf     intcon,GIE      ;enable interrupt
;
        call    Keyin           ;wait for 4 - digit code
        bcf     intcon,GIE      ;disable interrupt
        bsf     LED
        addlw   0x01
        btfss   status,Z        ;test result
        goto    Inact           ;code OK
        call    Siren           ;wrong code
        goto    Enin
;**************************************************************************
; end of PVK40 Example : Security alarm
        end
