;**************************************************************************
;
;                PVK40 Example : Infra-Red Receiver
;
; Note: UIR remote decoder, compatible with IRAMP/IRMAN
;
; Processor : PIC18F4520
; Clock : XT 3.2768 MHz
;
; Serial port : 9600 Bd, 8 db, parity none, 1 stop bit, flow control none
;
; On your PVK40 board :
; 1) Switch off all DIP switches except:
; 2) Switch on OSC 3.276M on S9
; 3) Switch on C0 IRF_RX on S11
; 4) Switch on C6 UART_TX on S11
; 5) Switch on C7 UART_RX on S11
; 6) Select COM or USB on JP4
; 7) If USB selected, insert UMS1 module into socket
;**************************************************************************
; Assembler directives :
        list    p = PIC18F4520  ; processor type
        __config 0x300001,0x01  ; configuration settings
        __config 0x300002,0x06
        __config 0x300003,0x00
        __config 0x300005,0x85
        __config 0x300006,0x80
        __config 0x300008,0x0F
        __config 0x300009,0xC0
        __config 0x30000A,0x0F
        __config 0x30000B,0xE0
        __config 0x30000C,0x0F
        __config 0x30000D,0x40
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P18f4520.INC>
; General Purpose Register definition :
rcdata  equ     0x00            ;received serial data
state   equ     0x01            ;bit0 : 1 = last received byte was 'I'
bcnt    equ     0x02            ;used in Sndbl,Cpy6b,Capuir subroutines
timel   equ     0x03            ;copy of tmr1l register
timeh   equ     0x04            ;copy of tmr1h register
ltmml   equ     0x05            ;used in Cpuir subroutine
ltmmh   equ     0x06            ;used in Cpuir subroutine
ltmsl   equ     0x07            ;used in Cpuir subroutine
ltmsh   equ     0x08            ;used in Cpuir subroutine
mask    equ     0x09            ;used in Cpuir subroutine
xmask   equ     0x0A            ;used in Cpuir subroutine
bits    equ     0x0B            ;used in Cpuir subroutine
cmptol  equ     0x0C            ;used in Cmp subroutine, tolerance
cmptl   equ     0x0D            ;used in Cmp subroutine
cmpth   equ     0x0E            ;used in Cmp subroutine
cmpres  equ     0x0F            ;used in Cmp subroutine
;
uircd   equ     0x40            ;IR received data
; bytes 0x41 - 0x45 are accessed via INDF
puircd  equ     0x46            ;repeated IR received data
; bytes 0x47 - 0x4B are accessed via INDF
; Bits definition :
#define IRFRX   portc,0         ;IR input
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
Start:  movlw   B'10111111'     ;pin RC6 is output (optional, RC6 is
        movwf   trisc,a         ; automaticaly configured as output if
        movlw   B'10010000'     ; UART is enabled)
        movwf   rcsta,a
        movlw   B'00100100'
        movwf   txsta,a
        movlw   B'01000000'
        movwf   baudcon,a
        movlw   0x14            ;9600 Bd
        movwf   spbrg,a
        movlw   B'10100001'     ;enable TMR1, 16bit - buffered
        movwf   t1con,a
        movf    rcreg,W,a       ;clear RCIF
        clrf    state,a
;
Main:   btfss   pir1,RCIF,a     ;test rx flag
        goto    Irtest
        movff   rcreg,rcdata    ;copy rcreg to rcdat
        movlw   'I'             ;if received data = 'I'
        xorwf   rcdata,W,a
        bz      Rci             ;branch to Rci
        movlw   'R'             ;if received data = 'R'
        xorwf   rcdata,W,a
        bz      Rcr             ;branch to Rcr
        bcf     state,0,a       ;else state,0 = 0
        goto    Irtest
Rci:    bsf     state,0,a       ;state,0 = 1
        goto    Irtest
Rcr:    btfss   state,0,a       ;if state,0 = 1
        goto    Irtest
        movlw   'O'             ;send "OK" to PC
        call    Send
        movlw   'K'
        call    Send
        bcf     state,0,a       ;state,0 = 0
;
Irtest: btfsc   IRFRX,a         ;test IR input
        goto    Main
        btfsc   pir1,TMR1IF,a   ;if timer overflow
        bcf     state,1,a       ;mark repeat data as old
        call    Cpuir           ;capture UIR
        andlw   0xF0            ;if result < 16
        bz      Rptt            ;then skip to Rptt
        bsf     state,1,a
        movlw   uircd
        call    Sndbl           ;send 6 bytes
        call    Cpy6b           ;copy uircd to puircd
        call    Clrtmr
        goto    Main
;
Rptt:   btfss   state,1,a       ;repeat?
        goto    Main            ;no
        movlw   puircd          ;yes
        call    Sndbl           ;send 6 bytes
        call    Clrtmr
        goto    Main
;--------------------------------------------------------------------------
; Subroutines :
Send:   movwf   txreg,a         ;send data
Send2:  btfss   txsta,TRMT,a
        bra     Send2
        return
;
Sndbl:  movwf   fsr0l,a         ;send 6-byte block
        clrf    fsr0h,a
        movlw   0x06
        movwf   bcnt,a
Sndbl2: movf    postinc0,W,a
        call    Send
        decfsz  bcnt,F,a
        bra     Sndbl2
        return
;
Clrtmr: clrf    tmr1h,a         ;clear timer1 register
        clrf    tmr1l,a
        bcf     pir1,TMR1IF,a
        return
;
Cpy6b:  clrf    fsr0h,a         ;copy 6 bytes from uircd:uircd+5
        clrf    fsr1h,a         ;to puircd:puircd+5
        movlw   uircd
        movwf   fsr0,a
        movlw   puircd
        movwf   fsr1,a
        movlw   0x06
        movwf   bcnt,a
Cpy6b2: movff   postinc0,postinc1
        decfsz  bcnt,f,a
        bra     Cpy6b2
        return
;
Cpuir:  movlw   0x06            ;capture UIR code
        movwf   bcnt,a
        movlw   uircd           ;clear uircd:uircd+5
        movwf   fsr0l,a
        clrf    fsr0h,a
Cpuir2: clrf    postinc0,a
        decfsz  bcnt,F,a
        bra     Cpuir2
;
        call    Clrtmr          ;clear timer 1
        call    Measm           ;wait for IRFRX = 1
        movf    timeh,W,a       ;test time
        andlw   0xFF
        btfsc   status,Z,a
        retlw   0x00            ;error, return 0
        call    Meass           ;wait for IRFRX = 0
        movf    timeh,W,a       ;test time
        andlw   0xF8
        btfss   status,Z,a
        retlw   0x00            ;error, return 0
;
        movlw   B'10000000'
        movwf   mask,a
        movlw   0x20
        movwf   cmptol,a
        movlw   timel
        movwf   fsr2l,a
        clrf    fsr2h,a
        clrf    bcnt,a
        clrf    bits,a
        clrf    ltmml,a
        clrf    ltmmh,a
        clrf    ltmsl,a
        clrf    ltmsh,a
;
Cpuir3: call    Measm           ;wait for IRFRX = 1
        movf    timeh,W,a       ;test time
        andlw   0xF8
        btfss   status,Z,a
        goto    Cpuir5          ;time-out, return
        movlw   ltmml
        movwf   fsr0l,a
        clrf    fsr0h,a
        call    Cmpt            ;compare with previous value
        andwf   mask,W,a
        movwf   xmask,a
        movlw   uircd
        addwf   bcnt,W,a
        movwf   fsr0l,a
        movf    indf0,W,a
        xorwf   xmask,W,a
        movwf   indf0,a
;
        movff   timel,ltmml     ;copy timeh:l to ltmmh:l
        movff   timeh,ltmmh
        incf    bits,F,a        ;increment number of bits
        bcf     status,C,a
        rrcf    mask,F,a
        bnc     Cpuir4          ;if bit C is set
        bsf     mask,7,a        ;mask = 0x80
        incf    bcnt,F,a        ;increment bcnt
        movlw   0x06
        cpfslt  bcnt,a          ;if bcnt >= 6
        clrf    bcnt,a          ;then bcnt = 0
;
Cpuir4: call    Meass           ;wait for IRFRX = 0
        movf    timeh,W,a       ;test time
        andlw   0xF8
        btfss   status,Z,a
        goto    Cpuir5          ;time-out, return
        movlw   ltmsl
        movwf   fsr0l,a
        call    Cmpt            ;compare with previous value
        andwf   mask,W,a
        movwf   xmask,a
        movlw   uircd
        addwf   bcnt,W,a
        movwf   fsr0l,a
        movf    indf0,W,a
        xorwf   xmask,W,a
        movwf   indf0,a
;
        movff   timel,ltmsl     ;copy timeh:l to ltmsh:l
        movff   timeh,ltmsh
        incf    bits,F,a        ;increment number of bits
        bcf     status,C,a
        rrcf    mask,F,a
        btfss   status,C,a      ;if bit C is set
        goto    Cpuir3
        bsf     mask,7,a        ;mask = 0x80
        incf    bcnt,F,a        ;increment bcnt
        movlw   0x06
        cpfslt  bcnt,a          ;if bcnt >= 6
        clrf    bcnt,a          ;then bcnt = 0
        goto    Cpuir3          ;loop
;
Cpuir5: movf    bits,W,a
        return
;
Gettm:  movff   tmr1l,timel     ;copy tmr1h:l to timeh:l
        movff   tmr1h,timeh
        return
;
Measm:  call    Gettm           ;wait for IRFRX = 1
        movlw   0x08
        cpfslt  timeh,a
        bra     Measm2
        btfss   IRFRX,a
        bra     Measm
Measm2: call    Clrtmr
        return
;
Meass:  call    Gettm           ;wait for IRFRX = 0
        movlw   0x08
        cpfslt  timeh,a
        bra     Meass2
        btfsc   IRFRX,a
        bra     Meass
Meass2: call    Clrtmr
        return
;
Comp:   movf    preinc0,w,a     ;compare two 16-bit register A,B
        cpfsgt  preinc1,a       ;A = *(FSR0,FSR0+1), B = *(FSR1,FSR1+1)
        bra     Comp3           ;A < B => cmpres = B'11100000'
        movf    postdec0,f,a    ;A = B => cmpres = B'00011000'
        movf    postdec1,f,a    ;A > B => cmpres = B'00000111'
Comp2:  movlw   B'11100000'
        movwf   cmpres,a        ;A < B
        return
Comp3:  movf    postdec0,w,a
        cpfslt  postdec1,a
        bra     Comp5
Comp4:  movlw   B'00000111'     ;A > B
        movwf   cmpres,a
        return
Comp5:  movf    indf0,w,a
        cpfseq  indf1,a
        bra     Comp6
        movlw   B'00011000'     ;A = B
        movwf   cmpres,a
        return
Comp6:  cpfsgt  indf1,a
        goto    Comp4
        goto    Comp2
;
Cmpt:   movlw   cmptl           ;compare 16-bit number A,B +/-cmptol
        movwf   fsr1l,a         ;A = *(FSR0,FSR0+1), B = *(FSR2,FSR2+1)
        clrf    fsr1h,a         ;A = B => return 0x00 else return 0xFF
        movff   postinc2,postinc1
        movff   postdec2,postdec1
        movf    cmptol,w,a      ;add tolerance
        addwf   postinc1,f,a
        movlw   0x00
        addwfc  postdec1,f,a
        bnc     $+6             ;if overflow,
        setf    postinc1,a      ;set INDF1
        setf    postdec1,a
        call    Comp            ;compare
        btfsc   cmpres,0,a
        retlw   0xFF            ;nonequal
;
        movff   postinc2,postinc1
        movff   postdec2,postdec1
        movf    cmptol,w,a      ;subtract tolerance
        subwf   postinc1,f,a
        movlw   0x00
        subwfb  postdec1,f,a
        bnn     $+6             ;if negative,
        clrf    postinc1,a      ;clear INDF1
        clrf    postdec1,a
        call    Comp            ;compare
        btfsc   cmpres,7,a
        retlw   0xFF            ;nonequal
        retlw   0x00            ;in tolerance
;**************************************************************************
; end of PVK40 Example : Infra-Red Receiver
        end
