;**************************************************************************
;
;                      PVK40 Example : Sound recording
;
; Processor : PIC16F877A
; Clock : XT 3.2768 MHz
;
; Serial port : 9600 Bd, 8 db, parity none, 1 stop bit, flow control none
;
; On your PVK40 board :
; 1) Insert 24FC515 into 24xx socket ***
; 2) Switch off all DIP switches except:
; 3) Switch on OSC 3.276M on S9
; 4) Switch on MIC on S9
; 5) Switch on B3 LED on S11
; 6) Switch on C6 UART_TX on S11
; 7) If USB selected, insert UMS1 module into socket
;
; *** Note: pin 3 (A2) must be tied to Vcc
;**************************************************************************
; Assembler directives :
        list    p = PIC16F877A  ; processor type
        __config 0x3F71         ; configuration setting
; EEPROM data :
        org     0x2100
        de      0x2C,0x52,0x49,0x46,0x46,0x24,0x00,0x01 ;wav file header
        de      0x00,0x57,0x41,0x56,0x45,0x66,0x6D,0x74 ;first byte
        de      0x20,0x10,0x00,0x00,0x00,0x01,0x00,0x01 ;is header length
        de      0x00,0x40,0x1F,0x00,0x00,0x40,0x1F,0x00
        de      0x00,0x01,0x00,0x08,0x00,0x64,0x61,0x74
        de      0x61,0x00,0x00,0x01,0x00,0x00,0x00,0x00
;--------------------------------------------------------------------------
; Special Function Register definition :
        #include <P16f877a.INC>
; General Purpose Register definition :
bcnt    equ     0x20            ;byte counting
intmp   equ     0x21            ;used in Input subroutine
ieaddr  equ     0x22            ;internal EEPROM address pointer
eaddrl  equ     0x23            ;I2C EEPROM address pointer, LSB
eaddrh  equ     0x24            ;I2C EEPROM address pointer, MSB
eedatb  equ     0x25            ;used in Wree and Rdee subroutines
reccon  equ     0x26            ;used during capture
bufptr  equ     0x27            ;pointer to 64B buffer
;  bytes 0x30 - 0x6F are used as 64B buffer
;  these bytes are accessed via indirect addressing
bcksta  equ     0x70            ;status register backup during interrupt
bckwreg equ     0x71            ;working register backup during interrupt
bckfsr  equ     0x72            ;fsr register backup during interrupt
;  bytes 0xB0 - 0xEF are used as 64B buffer
;  these bytes are accessed via indirect addressing
; Bits definition :
#define RDB0    reccon,0        ;block 0 data valid
#define RDB1    reccon,1        ;block 0 data valid
#define WRT0    reccon,2        ;writing block 0
#define WRT1    reccon,3        ;writing block 1
#define LED     portb,3         ;LED is on the RB3 pin
#define PB0     portd,0         ;pushbutton 0 - record
#define PB7     portd,7         ;pushbutton 7 - send to PC
;--------------------------------------------------------------------------
        org     0               ;program starts at address 0x000
        goto    Start           ;skip to start
;--------------------------------------------------------------------------
        org     0x04            ;interrupt vector
Interr: movwf   bckwreg         ;save wreg during interrupt
        swapf   status,w        ;save status during interrupt
        movwf   bcksta
        movf    fsr,w           ;save fsr register during interrupt
        movwf   bckfsr
        bcf     status,RP0      ;bank 0
        bcf     intcon,T0IF     ;clear flag
        movlw   0xA2            ;TMR0 period = 125usec
        movwf   tmr0
;
Adw:    btfsc   adcon0,GO       ;wait for A/D conversion to complete
        goto    Adw
        movf    bufptr,W        ;indirect addressing
        movwf   fsr
;
        btfsc   WRT0
        goto    Nwb0            ;writing block 0
        btfsc   WRT1
        goto    Nwb1            ;writing block 1
        btfss   RDB0
        goto    Swb0            ;start write to block 0
        btfss   RDB1
        goto    Swb1            ;start write to block 1
        goto    Intret          ;busy -> wait
;
Nwb0    movf    adresh,W        ;copy data to buffer
        movwf   indf
        incf    bufptr,F        ;increment pointer
        movlw   0x70
        xorwf   bufptr,W
        btfss   status,Z        ;if pointer = 0x70
        goto    Intret
        bcf     WRT0            ;stop writing to block 0
        bsf     RDB0            ;allow reading
        goto    Intret
;
Nwb1    movf    adresh,W        ;copy data to buffer
        movwf   indf
        incf    bufptr,F        ;increment pointer
        movlw   0xF0
        xorwf   bufptr,W
        btfss   status,Z        ;if pointer = 0xF0
        goto    Intret
        bcf     WRT1            ;stop writing to block 1
        bsf     RDB1            ;allow reading
        goto    Intret
;
Swb0:   movlw   0x30            ;first byte
        movwf   bufptr
        movwf   fsr
        movf    adresh,W        ;copy data
        movwf   indf
        incf    bufptr,F        ;increment
        bsf     WRT0            ;next time write to block 0
        goto    Intret
;
Swb1:   movlw   0xB0            ;first byte
        movwf   bufptr
        movwf   fsr
        movf    adresh,W        ;copy data
        movwf   indf
        incf    bufptr,F        ;increment
        bsf     WRT1            ;next time write to block 1
        goto    Intret
;
Intret: bsf     adcon0,GO       ;starts new A/D conversion
        movf    bckfsr,w        ;restore fsr register
        movwf   fsr
        swapf   bcksta,w        ;restore status register
        movwf   status
        swapf   bckwreg,f
        swapf   bckwreg,w       ;restore wreg
        retfie                  ;return and set GIE
;--------------------------------------------------------------------------
Start:  movlw   B'00000000'
        movwf   portb           ;clear RB3
        movlw   B'10000000'     ;serial port enabled, pin RC6 is
        movwf   rcsta           ;automaticaly configured as output
        bsf     status,RP0      ;bank 1
        movlw   B'11110111'     ;pin RB3 is output
        movwf   trisb
        movlw   0x14            ;9600 Bd
        movwf   spbrg
        movlw   B'00100110'     ;enable transmit
        movwf   txsta
        clrf    sspstat
        bsf     sspstat,SMP     ;disable slew rate control
        movlw   0x00
        movwf   sspadd          ;setup 1 MHz I2C clock (819kHz)
        clrf    sspcon2
        movlw   B'00000010'     ;pin RA5 is analog input,
        movwf   adcon1          ;porte pins are digital
        movlw   B'11011000'     ;prescaler assigned to WDT
        movwf   option_reg
        bcf     status,RP0      ;bank 0
        movlw   B'00101000'     ;I2C Master mode
        movwf   sspcon          ;SSPEN enabled
        bcf     pir1,SSPIF      ;clear SSP interrupt flag
        movlw   B'01100001'     ;enable A/D converter module
        movwf   adcon0
        movlw   B'00100000'
        movwf   intcon          ;unmask interrupt from TMR0
;
Main:   btfss   PB0             ;test pushbuttons
        goto    Rec             ;Rec button is pressed
        btfss   PB7
        goto    Ttpc            ;Transfer button is pressed
        goto    Main
;--------------------------------------------------------------------------
Rec:    bsf     LED
        bsf     adcon0,GO       ;starts A/D conversion
        clrf    eaddrh
        clrf    eaddrl
        movlw   B'00000000'
        movwf   reccon
        movlw   0xA2            ;TMR0 period = 125usec
        movwf   tmr0
        bcf     intcon,T0IF     ;clear timer0 overflow flag
        bsf     intcon,GIE      ;enable interrupt
;
Rec2:   btfsc   RDB0            ;wait for buffer full flag
        goto    Wrba
        btfsc   RDB1
        goto    Wrbb
        goto    Rec2
;
Wrba:   movlw   0x30            ;write bytes 0x30-0x6F to EEPROM
        call    Wree
        bcf     RDB0
        goto    Nxtb
Wrbb:   movlw   0xB0            ;write bytes 0xB0-0xEF to EEPROM
        call    Wree
        bcf     RDB1
Nxtb:   movlw   0x40            ;eaddrh:l = eaddrh:l + 64
        addwf   eaddrl,F
        btfss   status,C
        goto    Rec2
        incf    eaddrh,F
        btfss   status,Z        ;if eaddrh:l = 0x0000 (EEPROM full)
        goto    Rec2
;
        bcf     intcon,GIE      ;then disable interrupt
        bcf     LED             ;go to main loop
        goto    Main
;--------------------------------------------------------------------------
Ttpc:   movlw   0x00            ;transfer wav file to PC
        call    Rdie            ;read header length
        movwf   bcnt
        movlw   0x01            ;first byte is at address 0x01
        movwf   ieaddr
Thtpc:  movf    ieaddr,W        ;read from internal EEPROM
        call    Rdie
        call    Send            ;send byte
        incf    ieaddr,F
        decfsz  bcnt,F
        goto    Thtpc
;
        clrf    eaddrl          ;address = 0x0000
        clrf    eaddrh
Tetpc:  call    Rdee            ;read I2C EEPROM
        call    Send            ;send byte
        btfss   eaddrh,0        ;blinking LED
        bcf     LED
        btfsc   eaddrh,0
        bsf     LED
        incf    eaddrl,F        ;next byte
        movlw   0x00
        btfsc   status,Z
        movlw   0x01            ;increment eaddrh
        addwf   eaddrh,F
        btfss   status,C        ;if eaddrh overflow
        goto    Tetpc
        bcf     LED             ;go to main loop
        goto    Main
;--------------------------------------------------------------------------
; Subroutines :
Send:   bsf     status,RP0      ;bank 1
        btfss   txsta,TRMT
        goto    Send
        bcf     status,RP0      ;bank 0
        movwf   txreg           ;send data
        return
;
Rdie:   bsf     status,RP1      ;read from internal EEPROM; bank 2
        movwf   eeadr           ;Write address to read from
        bsf     status,RP0      ;bank 3
        bcf     eecon1,EEPGD    ;point to Data memory
        bsf     eecon1,RD       ;start read operation
        bcf     status,RP0      ;bank 2
        movf    eedata,W        ;W = EEDATA
        bcf     status,RP1      ;bank 0
        return
;
Sndsb:  bcf     pir1,SSPIF      ;this subroutine generates I2C start bit
        bsf     status,RP0      ;bank 1
        bsf     sspcon2,SEN     ;generate Start condition
        bcf     status,RP0      ;bank 0
Sndsb2: btfss   pir1,SSPIF      ;check if operation completed
        goto    Sndsb2
        return
;
Sndpb:  bcf     pir1,SSPIF      ;this subroutine generates I2C stop bit
        bsf     status,RP0      ;bank 1
        bsf     sspcon2,PEN     ;generate Stop condition
        bcf     status,RP0      ;bank 0
Sndpb2: btfss   pir1,SSPIF      ;check if operation completed
        goto    Sndpb2
        return
;
Sndrb:  bcf     pir1,SSPIF      ;this subroutine generates I2C restart bit
        bsf     status,RP0      ;bank 1
        bsf     sspcon2,RSEN    ;generate Restart condition
        bcf     status,RP0      ;bank 0
Sndrb2: btfss   pir1,SSPIF      ;check if operation completed
        goto    Sndrb2
        return
;
Output: bcf     pir1,SSPIF      ;this subroutine serial sends Wreg
        movwf   sspbuf
Outpt2: btfss   pir1,SSPIF      ;check if operation completed
        goto    Outpt2
        return
;
Input:  bcf     pir1,SSPIF      ;this subroutine serial reads to Wreg
        bsf     status,RP0      ;bank 1
        bsf     sspcon2,RCEN    ;initiate reception of byte
        bcf     status,RP0      ;bank 0
Input2: btfss   pir1,SSPIF      ;check if operation completed
        goto    Input2
        movf    sspbuf,w
        movwf   intmp
        bcf     pir1,SSPIF      ;clear SSP interrupt flag
        bsf     status,RP0      ;bank 1
        bsf     sspcon2,ACKEN   ;generate ACK/NO_ACK bit
        bcf     status,RP0      ;bank 0
Input3: btfss   pir1,SSPIF      ;check if operation completed
        goto    Input3
        movf    intmp,w
        return
;
Rdee:   call    Sndsb           ;read I2C EEPROM from address eaddrh:l
        movlw   B'10100000'     ;control byte - write
        btfsc   eaddrh,7        ;bit A15
        iorlw   B'00001000'
        call    Output
        bsf     status,RP0      ;bank 1
        btfss   sspcon2,ACKSTAT ;acknowledge pooling
        goto    Rdee2           ;ACK bit was sent
        bcf     status,RP0      ;bank 0, NO_ACK
        call    Sndpb           ;stop bit
        nop
        goto    Rdee
Rdee2:  bcf     status,RP0      ;bank 0
        movf    eaddrh,W
        andlw   B'01111111'
        call    Output          ;write address to EEPROM
        movf    eaddrl,W
        call    Output          ;write address to EEPROM
        call    Sndrb           ;restart bit
        movlw   B'10100001'     ;control byte - read
        btfsc   eaddrh,7        ;bit A15
        iorlw   B'00001000'
        call    Output
        bsf     status,RP0      ;bank 1
        bsf     sspcon2,ACKDT   ;select NO_ACK bit
        bcf     status,RP0      ;bank 0
        call    Input           ;read data
        movwf   eedatb
        call    Sndpb           ;stop bit
        movf    eedatb,W
        return
;
Wree:   movwf   fsr             ;write 64B to EEPROM
Wree2:  call    Sndsb           ;start bit
        movlw   B'10100000'     ;control byte - write
        btfsc   eaddrh,7        ;bit A15
        iorlw   B'00001000'
        call    Output
        bsf     status,RP0      ;bank 1
        btfss   sspcon2,ACKSTAT ;acknowledge pooling
        goto    Wree3           ;ACK bit was sent
        bcf     status,RP0      ;bank 0, NO_ACK
        call    Sndpb           ;stop bit
        nop
        goto    Wree2
Wree3:  bcf     status,RP0      ;bank 0
        movf    eaddrh,W
        andlw   B'01111111'
        call    Output          ;write address to EEPROM
        movf    eaddrl,W
        call    Output          ;write address to EEPROM
        movlw   0x40            ;64 bytes
        movwf   eedatb
Wree4:  movf    indf,W
        call    Output          ;write data to EEPROM
        incf    fsr,F           ;next address
        decfsz  eedatb,F
        goto    Wree4
        call    Sndpb           ;stop bit
        return
;**************************************************************************
; end of PVK40 Example : Sound recording
        end
