/*============================================================================

                        PVK40 Example : Blinking

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on OSC 3.276M on S9
 3) Switch on B3 LED on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/
#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //no modify this line

typedef unsigned int clock_t;

static bit button @ PORTBIT(PORTD,0);
static bit led @ PORTBIT(PORTB,3);

clock_t systimer;         /* systimer is incremented every 20ms */
clock_t now;
clock_t lap;
unsigned int period;

/*============================================================================

       Program

============================================================================*/

void interrupt timer0(void)
{
  T0IF=0;
  systimer++;
}

clock_t clock(void)        /* read systimer */
{
  clock_t temp;

  GIE=0;
  temp=systimer;
  GIE=1;
  return temp;
}


int main(void)
{
  TRISB=0xF7;
  /* initialisation of timer0 */
	
  PS2=1;  PS1=0;  PS0 = 1; /* select prescaler by 64 */
  T0CS = 0;                /* select timer0 */
  PSA=0;                   /* select prescaler for timer0 */
  TMR0 =0;                 /* preload timer0 */
  T0IE = 1;                /* interrupts are actived */
  GIE=1;                   /* interrupts are actived */
  period=25;               /* speed */
  /*--------------------------*/


  for(;;) {
    while (button==0)     /* wait for button */
      GIE=0;
    GIE=1;
    now=clock();
    if ((now-lap)>period) {
      lap+=period;
      led=!led;
    }
  }
}
