/*============================================================================

                        PVK40 Example : Display

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on OSC 3.276M on S9
 3) Switch on B0,1,2,4 DISP on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/
#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //no modify this line

volatile bit button0 @ PORTBIT(PORTD,0);
volatile bit button1 @ PORTBIT(PORTD,1);
volatile bit button2 @ PORTBIT(PORTD,2);
volatile bit button3 @ PORTBIT(PORTD,3);
volatile bit button4 @ PORTBIT(PORTD,4);
volatile bit button5 @ PORTBIT(PORTD,5);
volatile bit button6 @ PORTBIT(PORTD,6);
volatile bit button7 @ PORTBIT(PORTD,7);

char segment[8] = {0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80}; /*12345678*/

unsigned char buttons @ (unsigned char)&PORTD;


/*============================================================================

       Program

============================================================================*/

int main(void)
{
  unsigned char i;
  unsigned char temp;

  TRISB=0x00;
  PORTB=0xFF;

  for(;;) {
    TRISD=0xFF;      /* PORTD input */
    asm("nop");      /*need to wait before reading to have the right value*/
    asm("nop");
    temp=buttons;
    TRISD=0x00;

    for(i=0;i<8;i++) {
      if (!(temp&1)) {
        PORTD=segment[i];    /*show i+1 on the right display*/
        PORTB=0xFE;
        break; 
      }
      temp>>=1;
    }

    if (i==8) PORTB=0xFF;
  }
}
