/*============================================================================

                        PVK40 Example : Thermometer

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on OSC 3.276M on S9
 3) Switch on TEMP on S9
 4) Switch on B0,1,2,4 DISP on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>
#include <math.h>

/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/

#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //don't modify this line

#define R17 2000           /* constants for calcultemp function */
#define K1 -0.000015 
#define K2 0.000077
#define K3 0.000039
#define alpha 7.88e-3

const unsigned char digit[] = {0xFF,0xFE,0xFD,0xFB,0xEF};
const unsigned char segment[]={0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,
                               0x80,0x90,0x88,0x83,0xC6,0xA1,0x86,0x8E};
unsigned char dispbuffer[4];
unsigned char disppos=0;
unsigned char dispen=0;


/*============================================================================

       Program

============================================================================*/

void dispinit(void)
{
  TRISD=0x00;
  TRISB=0xE8;
}

void disprefresh(void)      /*show the value of an integer j */
{                           /* on one p of the 7 segments display */
  PORTB=digit[0];
  if (!dispen)
    return;
  PORTD=dispbuffer[disppos];
  PORTB=digit[disppos+1];
  disppos=(++disppos)&3;
}

void interrupt timer0(void)
{
	disprefresh();
	T0IF=0;
}

void timer0init(void)
{
 /* initialisation of timer0; */
	PS2=0; PS1=0; PS0=0; /* select prescaler by 2 */
	T0CS=0; /* select timer0 */
	PSA=0;  /* select prescaler for timer0 */
	TMR0=0; /* preload timer0 */
	T0IE=1; /* interrupts are actived */
}

void delayus(unsigned int us)
{
  for (;us--;);
}

void adcinit(void)
{
  /* initialisation of AD converter */
  TRISA = 0xFF;	/* all pin of portA are inputs */
  ADCON1 = 0x80;  /*config in 5/0 */
  ADCON0 = 0x41; /* config the converter  */
  delayus(100);
}

unsigned int adcsample(void)
{

  for (ADCON0|=0x04; ADCON0&0x04; );  /* start and wait for done */

  return ADRESL|(ADRESH<<8);
}

void dispthermo(int value)
{
  dispen=0;
  if (value<0) {
    dispbuffer[1]=0x94;  /* degree with minus */
    value=-value;
  }
  else
    dispbuffer[1]=0x9C; /* degree */
  dispbuffer[3]=segment[(value/10)%10];  /* value of temp */
  dispbuffer[2]=segment[value%10];       /* value of temp */
  dispbuffer[0]=0xC6;            /* caractere C */
  dispen=1;
}

int calcultemp(unsigned int value)
{
  float kt;
  float t;

  kt=((float)value*R17)/(1024-value)/2000;
  t=25 + (sqrt(K1+K2*kt)-alpha) / K3;
  return (int)t;
}

int main(void)
{
  unsigned int value;

  dispinit();
  timer0init();
  adcinit();
  GIE=1;  /* interrupts are actived */

  for(;;) {
    value=adcsample();
    dispthermo(calcultemp(value));
    delayus(2000);
  }
}
