/*============================================================================

     lcd.c - handling routines for LCD module MC1602

 Board : PVK40
 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

============================================================================*/

#include <pic.h>
#include "lcd.h"

#define cbi(X,Y)		((X)&=~(1<<(Y)))
#define sbi(X,Y)		((X)|=1<<(Y))

unsigned char PORTD_LATCH;
unsigned char PORTE_LATCH;

//approximate delay loop (1 instruction cycle lasts cca 1 us)
void MicroDelay(unsigned short us)
{
  while (us--);
}

void DispEn(void)
{
  PORTE=PORTE_LATCH|=0x04;
  PORTE=PORTE_LATCH&=0xfb;
}

void DispWr(unsigned char c, unsigned char isdata)
{
  PORTE_LATCH=PORTE;
  if (isdata)
    PORTE=PORTE_LATCH|=0x01;
  else
    PORTE=PORTE_LATCH&=0x04;

  PORTD=c;
  DispEn();
}

void DispWrByte(unsigned char c, unsigned char isdata, unsigned short delay_us)
{
  DispWr(c,isdata);
  MicroDelay(delay_us);
}

/*=====================================================*/
/* Display hardware initialisation */
void DispHwInit(void)
{
  /* RS */
  cbi(PORTE,0);
  cbi(TRISE,0);

  /* read/write */
  cbi(PORTE,1);
  cbi(TRISE,1);

  /* enable */
  cbi(PORTE,2);
  cbi(TRISE,2);

  /* data */
  TRISD=0;

  /*
  force the display to go into 8-bit mode
  since it might be in 4-bit mode and unsynchronized, thus we should better
  wait wait for possible instruction to finish after first DispWr()
  */
  DispWr(0x30,0);
  MicroDelay(2000);
  DispWr(0x30,0);
  MicroDelay(50);
  DispWr(0x30,0);
  MicroDelay(50);

  DispWrByte(0x38,0,50); //8-bit mode, 2 lines, 5x8 matrix

  DisplayClear();
  DisplayOnOff(DISPLAY_ONOFF|CURSOR_LINE|CURSOR_BLINK); //restore display state

  DispWrByte(0x06,0,50); //entry mode - move to right, no display shift
}

