/*============================================================================

          PVK40 Example : Infra-red Reflective photoelectric sensor

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on OSC 3.276M on S9
 3) Switch on B3 LED on S11
 4) Switch on B5 IRF_TX on S11
 5) Switch on C0 IRF_RX on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/

#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //don't modify this line

static bit ir_rx @ PORTBIT(PORTC,0);
static bit ir_tx @ PORTBIT(PORTB,5);
static bit led @ PORTBIT(PORTB,3);


/*============================================================================

       Program

============================================================================*/

void timer0init(void)
{ /* initialisation of timer0; */
	T0CS=0; /* select timer0 clock */
	PSA=1;  /* select prescaler for wdt */
	TMR0|=0xEF; /* preload timer0 */
}

int main(void)
{
  TRISB=0xD7;
  timer0init();

  for(;;) {
    led=ir_rx;
    ir_tx=1;
    asm("nop");
    ir_tx=0;
    while(!T0IF) ; /* wait for timer0 overflow */
    TMR0|=0xEF; /* preload timer0 */
    T0IF=0;  /* clear flag */
  }
}
