/*============================================================================

                        PVK40 Example : UART - Rx

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 Serial port : 9600 Bd, 8 db, parity none, 1 stop bit, flow control none

 On your PVK40 board set DIP switches according to following way:
 1) Switch off all DIP switches except:
 2) Switch on OSC 3.276M on S9
 3) Switch on C6 UART_RX on S11
 4) Switch on B0,1,2,4 DISP on S11
 5) Select COM or USB on JP4
 6) If USB selected, insert UMS1 module into socket

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/
#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //no modify this line

char digit[5] = {0xFF,0xFE,0xFD,0xFB,0xEF};
const unsigned char segment[]={0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,
                               0x80,0x90,0x88,0x83,0xC6,0xA1,0x86,0x8E};
unsigned char buffer;

/*============================================================================

       Program

============================================================================*/

void portinit(void)
{
  PORTD=0xFF;
  TRISB=0xE8;
  TRISD=0x00;
}

void uart_init(void)
{  /* serial port initialisation */
  RCSTA=0x90; /* receive enabled */
  SPBRG=0x14; /* 9600Bd */
  TXSTA=0x06; /* TX disabled */
  PEIE=1; /* peripheral interrupts enabled */
  RCIE=1;
  buffer=RCREG; /* clear RCIF flag */
  buffer=RCREG;
}

void interrupt uart_rx(void)
{
  buffer=RCREG; /* RCIF is cleared by hardware */
}

void delay(void)
{
  unsigned char n;
  for (n=0xFF;n>0;n--);
}

/* dispdigit(j,p) - shows the value of an integer j
   on one p of the 7 segments display            */ 
void dispdigit(int j, int p)
{
  PORTB=digit[0];
  PORTD=segment[j];
  TRISD=0x00;
  PORTB=digit[p];
}

void main(void)
{
  unsigned char buffer2;

  portinit();
  uart_init();

  while(!RCIF) ; /* wait for first byte */
  GIE=1;         /* interrupts are actived */
  asm("nop");

  for(;;) {
    GIE=0;         /* interrupts are disabled */
    buffer2=buffer;
    GIE=1;         /* interrupts are actived */
    dispdigit(buffer2&0x0F,1);
    delay();
    dispdigit(buffer2>>4,2);
    delay();
  }
}
